#========================================================================================
# package      DE_EPAGES::ExternalPayment::UI::PaymentMethodExternalPayment
# base         DE_EPAGES::Payment::UI::PaymentMethod
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for external payment method fetures
#========================================================================================
package DE_EPAGES::ExternalPayment::UI::PaymentMethodExternalPayment;
use base DE_EPAGES::Payment::UI::PaymentMethod;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );

use strict;


#========================================================================================
# function     ViewSettingsMultiPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettingsMultiPayment( $Servlet, $PaymentMethodClassName, $MethodAttributeName );
#----------------------------------------------------------------------------------------
# description  helper for multi payment MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# input        $PaymentMethodClassName | payment method class name | string
# input        $MethodAttributeName | method attribute name | string
#========================================================================================
sub ViewSettingsMultiPayment {
    my $self = shift;
    my ($Servlet, $PaymentMethodClassName, $MethodAttributeName, $aFilterMethodsId) = @_;

    $Servlet->vars('SplitGatewayMethods_MaxRows',$self->GetSplitGatewayMethodsMaxRow());
    $Servlet->vars('SplitGatewayMethods', $self->GetSplitGatewayMethods());

    if (isValid($aFilterMethodsId)) {
        $self->_filterAndSetBatchPaymentMethods($Servlet, $aFilterMethodsId);
    }

    my $PaymentMethod = $Servlet->object();
    $Servlet->vars('CanReplicatePayment', $PaymentMethod->usableAtStorefront());

    my $Shop = $Servlet->object()->getSite();
    my $PaymentMethods = $Shop->folder('PaymentMethods')->children;

    foreach my $PaymentMethod (@$PaymentMethods) {
        next unless $PaymentMethod->class()->alias() eq $PaymentMethodClassName;
        my $Method = $PaymentMethod->get($MethodAttributeName);
        next unless isValid($Method);
        $Servlet->vars('ExistingMultiPaymentMethod_' . $PaymentMethod->get($MethodAttributeName), 1);
    }

    return;
}


#========================================================================================
# function     NewMultiPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       NewMultiPaymentMethod( $Servlet );
#----------------------------------------------------------------------------------------
# description  Payment method replication with a different method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub NewMultiPaymentMethod {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $PaymentMethod = $Servlet->object();
    my $Values = $Servlet->form->formValues();

    my $aSplitGatewayMethods = $self->GetSplitGatewayMethods();

    # Go through the payment methods array and generate a hash with key = ID and value the payment method.
    my %hSplitGatewayMethods = map { ($_->{'ID'}) => $_ } @$aSplitGatewayMethods;

    # We build an array with the selected methods in the settings page
    my @aSelectedSplitGatewayMethods = map { /^aMultiPaymentMethod_(.*)/x } grep { /^aMultiPaymentMethod_/x } keys %$Values;

    # We check the selected methods
    my $LeftPaymentMethods = $self->_checkNewPaymentMethods($Servlet, $PaymentMethod, \@aSelectedSplitGatewayMethods, \%hSplitGatewayMethods);

    if ( $LeftPaymentMethods > 0 ) {
        splice(@aSelectedSplitGatewayMethods, -1 * $LeftPaymentMethods);
        $Form->addFormError({
                'Form'  => 'PaymentMethod',
                'Reason'=> 'FEATURE_LIMIT_EXCEEDED',
                'Name'  => 'FEATURE_LIMIT_EXCEEDED'
            });
    }

    # We create the diferent selected payment method and add them to an array
    my @newPaymentMethods;
    foreach my $SplitGatewayMethod (@aSelectedSplitGatewayMethods) {
        push(@newPaymentMethods,
             $self->NewPaymentMethod($Servlet, $PaymentMethod, $SplitGatewayMethod,
                    ($hSplitGatewayMethods{$SplitGatewayMethod}->{'UIName'} //
                            $hSplitGatewayMethods{$SplitGatewayMethod}->{'Name'}),
                    $hSplitGatewayMethods{$SplitGatewayMethod}->{'LogoName'}
             )
        );
    }

    $PaymentMethod->getSite()->child('PaymentMethods')->renumberChildren;
    $Servlet->vars('NewPaymentMethods',\@newPaymentMethods);

    $Form->executeFormError();

    return;
}


#========================================================================================
# function     GetSplitGatewayMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSplitGatewayMethods = $self->GetSplitGatewayMethods( );
#----------------------------------------------------------------------------------------
# description  It provides the multi-payment structure of grouped payment methods.
#----------------------------------------------------------------------------------------
# return       $aSplitGatewayMethods | hash with split gateway methods | ref.array
#========================================================================================
sub GetSplitGatewayMethods {
    my $self = shift;

    Error('function for Split Gateway Method structure not defined');
}


#========================================================================================
# function     GetSplitGatewayMethodsMaxRow
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MaxRow = $self->GetSplitGatewayMethodsMaxRow( );
#----------------------------------------------------------------------------------------
# description  It provides the maximum number of payment methods of any group.
#----------------------------------------------------------------------------------------
# return       $MaxRow | maximum number of payment methods per group| integer
#========================================================================================
sub GetSplitGatewayMethodsMaxRow {
    my $self = shift;

    Error('function for Split Gateway Method Max Row number not defined');
}


#========================================================================================
# function     NewPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       NewPaymentMethod( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName );
#----------------------------------------------------------------------------------------
# description  It creates a new payment method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# input        $PaymentMethod | payment method object | object
# input        $SplitGatewayMethod | payment method | string
# input        $MethodName | method name | string
# input        $LogoName | logo name | string
# return       $NewPaymentMethod | payment method | object
#========================================================================================
sub NewPaymentMethod {
    my $self = shift;
    my ( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName ) = @_;

    Error('new payment method function not defined');
}


#==============================================================================
sub _checkNewPaymentMethods{
    my $self = shift;
    my ($Servlet, $PaymentMethod, $aSelectedSplitGatewayMethods, $hSplitGatewyMethods) = @_;

    my $Shop = $PaymentMethod->getSite();
    my $Folder = $Shop->folder('PaymentMethods');
    my $Form = $Servlet->form();
    my $ExecuteError = 0;

    if (scalar @$aSelectedSplitGatewayMethods) {
        foreach my $SplitGatewayMethod (@$aSelectedSplitGatewayMethods) {
            if ($Folder->existsChild($hSplitGatewyMethods->{$SplitGatewayMethod}->{'Name'})) {
                $Servlet->vars('aMultiPaymentMethod_' . $SplitGatewayMethod, 1);
                $ExecuteError = 1;
                $Form->addFormError({
                    'Form'  => 'PaymentMethod',
                    'Reason'=> 'OBJECT_EXISTS',
                    'Name'  => 'NewMultiPaymentMethod' . $SplitGatewayMethod
                });
            }
        }
    } else {
        $Form->addFormError({
            'Form'  => 'PaymentMethod',
            'Reason'=> 'NOTHING_SELECTED',
            'Name'  => 'NOTHING_SELECTED'
        });
    }

    my $LeftPaymentMethods = $Shop->featureCurrentValue('PaymentMethods') + scalar(@$aSelectedSplitGatewayMethods) - $Shop->featureMaxValue('PaymentMethods');
    if ( $LeftPaymentMethods >= scalar(@$aSelectedSplitGatewayMethods) ){
        $ExecuteError = 1;
        $Form->addFormError({
                'Form'  => 'PaymentMethod',
                'Reason'=> 'FEATURE_LIMIT_EXCEEDED',
                'Name'  => 'FEATURE_LIMIT_EXCEEDED'
            });
    }
    if ( $ExecuteError == 1 ){
        $Form->executeFormError();
    }

    return $LeftPaymentMethods;
}

sub _filterAndSetBatchPaymentMethods {
    my $self = shift;
    my ($Servlet, $aFilterMethodsId) = @_;

    my @FilterMethodsId = @{$aFilterMethodsId};

    my @AllMethods = $self->GetSplitGatewayMethods();

    my @FilteredMethods;
    my $CountAllMethods = scalar(@{$AllMethods[0]});
    my $CountGroupMethods = 0;
    my $MaxMethodsRows = 0;

    for (my $i = $CountAllMethods - 1; $i >= 0; $i--) {

        if ($AllMethods[0][$i]->{'isGroup'}) {
            if ($CountGroupMethods) {
                $AllMethods[0][$i]->{'GrpCount'} = $CountGroupMethods;
                push(@FilteredMethods, $AllMethods[0][$i]);
                $MaxMethodsRows = ($CountGroupMethods > $MaxMethodsRows) ? $CountGroupMethods : $MaxMethodsRows;
            }
            $CountGroupMethods = 0;
            next;
        }

        if ($AllMethods[0][$i]->{'ID'} ~~ @FilterMethodsId) {
            push(@FilteredMethods, $AllMethods[0][$i]);
            $CountGroupMethods++;
        }

    }

    @AllMethods = [ reverse(@FilteredMethods) ];

    $Servlet->vars('SplitGatewayMethods_MaxRows', $MaxMethodsRows);
    $Servlet->vars('SplitGatewayMethods', @AllMethods);
}

1;