#==============================================================================
# package      DE_EPAGES::Facebook::UI::Facebook
# state        private
#==============================================================================
package DE_EPAGES::Facebook::UI::Facebook;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use LWP::UserAgent;
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Core::API::Url qw (ParseQueryString BuildParameterString);
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject);

use constant MAPPINGSERVER_TIMEOUT => 30; # seconds

#========================================================================================
# function     SaveFacebookApp
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->SaveFacebookApp($Servlet);
#----------------------------------------------------------------------------------------
# description  function was called to transfer mapping settings to the mappingserver
#               Http request is used to save parameters on the mappingserver
#========================================================================================
sub SaveFacebookApp {
    my $self    = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $System = LoadRootObject();

    # fetch data from the mapping server for saving
    my $MappingServerAndPort = $System->get('FacebookMappingServer'); # http://ep-support.epages.de:8081
    my $ShopGUID = $Shop->get('GUID');

    my $hFormValues = $Servlet->form->form($Shop, 'SaveFacebookApp');
    my $PageID = $hFormValues->{'FacebookPageID'};

    unless ($PageID || $Servlet->params('ACCEPT')) {
        $Servlet->vars('DELETE', 1);
        return;
    }
    my $OldPageID = $Servlet->params('OldFacebookPageID');

    $Servlet->form->addFormError({
        Reason => 'INVALID_FORM',
        Name   => 'FacebookPageID',
    }) if $PageID && $PageID !~ /^\d+$/;
    $Servlet->form->executeFormError;

    my $hParams;

    # parameters for the mapping server
    $hParams->{'ObjectPath'} = '/';
    $hParams->{'ViewAction'} = 'SaveFacebookMapping';

    # parameters to save on the mapping server
    $hParams->{'PageID'} = $PageID ? $PageID : $ShopGUID; # if pageid is empty, use the shopguid as a delete indicator
    $hParams->{'ShopGUID'} = $ShopGUID;
    $hParams->{'WebServerScriptUrl'} = $Shop->get('WebServerScriptUrl');
    $hParams->{'WebServerScriptUrlSSL'} = $Shop->get('WebServerScriptUrlSSL');
    $hParams->{'ShopPath'} = $Shop->get('Path');
    $hParams->{'ProviderID'} = $System->get('ProviderID');
    $hParams->{'StoreName'} = $System->get('StoreName');

    # build a request and fetch data by GUID
    my $UserAgent = LWP::UserAgent->new;
    $UserAgent->agent('ePages/6 ' . $UserAgent->agent());
    $UserAgent->timeout(MAPPINGSERVER_TIMEOUT);
    my $response = $UserAgent->post($MappingServerAndPort, $hParams);

    unless ($response->status_line eq '200 OK') {
        $Servlet->vars('HTTP_ERROR',1);
        Warning('HTTPResponseFailure',{'StatusLine' => $response->status_line});
    }
    if ($response->content eq 'SAVED') {
        if($PageID && $PageID ne $OldPageID) {
            $Servlet->vars('FB_SAVED', 1);
        }
    }

    if($hFormValues->{'FacebookPageID'} && $hFormValues->{'FacebookPageID'} ne '') {
        $Shop->set({ 'IsFacebookPageUsed' => 1 });
    } else {
        $Shop->set({ 'IsFacebookPageUsed' => 0 });
    }

    # save other settings
    $Servlet->object->getSite()->invalidatePageCache;
    return $self->SUPER::Save($Servlet);

}


# was called if the merchant display the Facebook App page in the mbo
# fetch data from the mapping server for displaying

#========================================================================================
# function     ViewFacebookApp
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->ViewFacebookApp($Servlet);
#----------------------------------------------------------------------------------------
# description  function was called to get the mapping for this shopguid from the mappingserver
#               Http request is used to get parameters from the mappingserver
#========================================================================================
sub ViewFacebookApp {
    my $self    = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $System = LoadRootObject();


    my $MappingServerAndPort =  $System->get('FacebookMappingServer'); # http://ep-support.epages.de:8081
    my $ShopGUID = $Shop->get('GUID');

    my $hParams;
    # parameters for the mapping server
    $hParams->{'ObjectPath'} = '/';
    $hParams->{'ViewAction'} = 'GetFacebookMapping';

    # parameter for fetching data from the mapping server
    $hParams->{'ShopGUID'} = $ShopGUID;

    # build a request and fetch data by GUID
    my $UserAgent = LWP::UserAgent->new;
    $UserAgent->agent('ePages/6 ' . $UserAgent->agent());
    $UserAgent->timeout(MAPPINGSERVER_TIMEOUT);

    my $URL = $MappingServerAndPort.'?'.BuildParameterString($hParams);

    my $response = $UserAgent->get($URL);

    if ($response->is_success){

        my $hValues = ParseQueryString($response->content,'utf-8');
        $Servlet->vars('FB_'.$_, $hValues->{$_}[0]) foreach keys(%$hValues) ;

        # create a datetime object
        if (exists $hValues->{'CreationDate'}) {
            my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
            my $DateTime     = $Formatter->parse_datetime( $hValues->{'CreationDate'}[0], 'datetime','%Y-%m-%dT%H:%M:%S' );
            $Servlet->vars('FB_CreationDate',$DateTime);
        }
    } else {
        Warning('HTTPResponseFailure',{'StatusLine' => $response->status_line});
        $Servlet->vars('FB_PageID',undef);
        $Servlet->vars('FB_ApplicationID',undef);
    }

    return $self->SUPER::View($Servlet);
}


# storefront call for displaying facebook products on facebook

#========================================================================================
# function     ViewFacebookProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->ViewFacebookProducts($Servlet);
#----------------------------------------------------------------------------------------
# description  function was called from facebook to display products
#
#========================================================================================
sub ViewFacebookProducts {
    my $self    = shift;
    my ($Servlet) = @_;

    # view cached PageTypes
    return $self->SUPER::ViewCached($Servlet);

}



1;



