#========================================================================================
# §package      DE_EPAGES::FacebookBusiness::API::Object::RemoteFacebook
# §state        public
#----------------------------------------------------------------------------------------
# §description  object for REST requests for OAuth calls and graph API
#========================================================================================
package DE_EPAGES::FacebookBusiness::API::Object::RemoteFacebook;

use strict;
use HTTP::Request;
use JSON::XS qw ( decode_json encode_json );
use LWP::UserAgent;
use Encode qw ( encode );
use DE_EPAGES::Core::API::Error          qw( Error ExistsError ResetError );
use DE_EPAGES::Core::API::Log            qw( GetCommunicationLog );
use DE_EPAGES::Core::API::TestParameter  qw( TestString TestObject );
use DE_EPAGES::Core::API::Url            qw( BuildParameterString );
use DE_EPAGES::Core::API::Warning        qw( Warning );
use DE_EPAGES::Object::API::Factory      qw( InsertObject LoadRootObject );
use DE_EPAGES::FacebookBusiness::API::Constants qw ( FACEBOOK_GRAPH_URL FACEBOOK_API_VERSION );

#ERRORDEFINITION
sub ErrorDefinition { return {}
}
#ERRORDEFINITION END

#========================================================================================
sub new {
    my $class = shift;
    my %options = @_;

    my $Env = $options{'Environment'};

    $Env = LoadRootObject()->get('FacebookBusinessEnvironment') unless defined $Env;
    Error('FacebookBusinessEnvironment') if (not $Env);
    $options{'Environment'} = $Env;
    $options{'AppAccessToken'} = undef;

    return bless \%options, $class;
}

#========================================================================================
# §function     getOAuthTokensByAuthCode
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->getOAuthTokensByAuthCode($AuthCode);
#----------------------------------------------------------------------------------------
# §description  Get the OAuth token by the browsers auth code.
#----------------------------------------------------------------------------------------
# §input        $AuthCode | auth code for fetching user token | string
# §return       $result | with following keys:
#               <ul>
#                   <li>access_token - access token for API calls - string
#                   <li>expires_in - access token exiration time - string
#                   <li>token_type - token type - string
#               </ul> | ref.hash
#========================================================================================
sub getOAuthUserTokenByAuthCode {
    my $self = shift;
    my ($BrowserAuthCode) = @_;

    TestString("AuthCode", $BrowserAuthCode);

    my $hCredentials = $self->appCredentials();

    my $hParams = {};
    $hParams->{'Header'} = {};
    $hParams->{'Query'}  = { 
                             'client_id'     => $hCredentials->{'AppID'},
                             'client_secret' => $hCredentials->{'AppSecret'},
                             'redirect_uri'  => $self->environment->get('FacebookServerRedirectURI'),
                             'code'          => $BrowserAuthCode,
                          };

    my $Route = '/oauth/access_token';
    my $hResponse = $self->sendRESTRequest('GET', $self->graphURL.$self->apiVersion, $Route, $hParams);

    return $hResponse;
}

#========================================================================================
# §function     getAppAccessToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $hToken = $self->getAppAccessToken();
#----------------------------------------------------------------------------------------
# §description  Get the app access token by app credentials (do not save this token!).
#----------------------------------------------------------------------------------------
# §return       $result | with following keys:
#               <ul>
#                   <li>access_token - access token for API calls - string
#                   <li>token_type - token type - string
#               </ul> | ref.hash
#========================================================================================
sub getAppAccessToken {
    my $self = shift;

    # return an existing access token in the correct structure
    if (defined $self->{'AppAccessToken'}) {
        return {
                'access_token' => $self->{'AppAccessToken'},
                'token_type'   => 'bearer'
               };
    }

    my $hCredentials = $self->appCredentials();
    my $hParams = {};
    $hParams->{'Header'} = {};
    $hParams->{'Query'}  = { 
                             'client_id'     => $hCredentials->{'AppID'},
                             'client_secret' => $hCredentials->{'AppSecret'},
                             'grant_type'    => 'client_credentials',
                          };

    my $Route = '/oauth/access_token';
    my $hResponse = $self->sendRESTRequest('GET', $self->graphURL.$self->apiVersion, $Route, $hParams);

    $self->setAppAccessToken($hResponse->{'access_token'});

    return $hResponse;
}

#========================================================================================
# §function     setAppAccessToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->setAppAccessToken($AccessToken);
#----------------------------------------------------------------------------------------
# §description  Set the app access token by app credentials on permanetely.
#----------------------------------------------------------------------------------------
# §input        $AccessToken | app access token | string 
#========================================================================================
sub setAppAccessToken {
    my $self = shift;
    my ($AppAccessToken) = @_;

    $self->{'AppAccessToken'} = $AppAccessToken;
}

#========================================================================================
# §function     debugOAuthUserToken
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       my $hDebugInfo = $self->debugOAuthUserToken();
#----------------------------------------------------------------------------------------
# §description  Debug the OAuth user token for getting informations.
#----------------------------------------------------------------------------------------
# §input        $OAuthUserToken | user token | string 
# §return       $hDebugInfo | token debug info | ref.hash
#========================================================================================
sub debugOAuthUserToken {
    my $self = shift;
    my ($OAuthUserToken) = @_;

    TestString("OAuthUserToken", $OAuthUserToken);

    my $AppAccessToken = $self->getAppAccessToken()->{'access_token'};

    my $hParams = {};
    $hParams->{'Header'} = {};
    $hParams->{'Query'}  = { 
                             'input_token'  => $OAuthUserToken,
                             'access_token' => $AppAccessToken,
                          };

    my $Route = '/debug_token';
    my $hResponse = $self->sendRESTRequest('GET', $self->graphURL.$self->apiVersion, $Route, $hParams);

    return $hResponse;
}

#========================================================================================
# §function     sendRESTRequest
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Shop->sendRESTRequest($Method, $APIUrl, $Route, $hParams);
# §syntax       $Shop->sendRESTRequest('GET', 'https://graph.facebook.com/v6.0',
#                                      /oauth/access_token , $hParams);
#----------------------------------------------------------------------------------------
# §description  Sends a request to the Facebook API server with the given parameter.
#               Throws API_CALL_ERROR if the request was not successfull.
#----------------------------------------------------------------------------------------
# §input        $Method | request method  | string
# §input        $APIUrl | API url  | string
# §input        $Route | route to object | string
# §input        $hParams | with following keys:
#               <ul>
#                   <li>Header - http header parameters (optional) - ref.hash
#                   <li>Query - query parameters for call (optional) - ref.hash
#                   <li>Content - payload for call (optional) - ref.hash
#                   <li>IgnoreCodes - ignore this response codes (do not produce errors) - ref.array
#               </ul> | ref.hash
# §return       $Response | JSON decoded response or undef | ref.hash
#========================================================================================
sub sendRESTRequest {
    my $self = shift;
    my ($Method, $APIUrl, $Route, $hParams) = @_;

    $Method //= 'GET';
    TestString( 'Method', $Method );
    TestString( 'APIUrl', $APIUrl );
    TestString( 'Route', $Route );

    # encode content
    my $EncodedContent;
    $EncodedContent = encode('UTF-8',encode_json($hParams->{'Content'})) if defined $hParams->{'Content'};

    # build the correct request URL by adding route and query params
    my $RequestURL = $self->_buildRequestURL($APIUrl, $Route, $hParams->{'Query'});
    my $Request = HTTP::Request->new($Method => $RequestURL);

    my $hHTTPHeader = { #'Authorization' => 'Bearer ' . $self->oAuthToken(),
                        'Content-Type'  => 'application/json',
                        'Accept'        => 'application/json',
                        'Accept-Charset'=> 'utf-8',
                     };

    # add additional header values
    foreach my $Key (keys(%{  $hParams->{'Header'} })) {
        $hHTTPHeader->{$Key} = $hParams->{'Header'}->{$Key};
    }
    $Request->header(%$hHTTPHeader);

    # adds the content and send request
    $Request->content($EncodedContent) if defined $EncodedContent;

    my $LogCategory = $self->_getLogCategory();
    $self->logCommunication($LogCategory, $Request);
    my $Response = LWP::UserAgent->new->request($Request);
    $self->logCommunication( $LogCategory, $Response);

    # error handling
    unless ($Response->is_success) {


        # try to json decode the message for having a error hash
        my $DecodedMessage;
        eval { $DecodedMessage = decode_json($Response->decoded_content); };
        ResetError() if ExistsError();

        my $ErrorObject = {
            'Method'  => $Method,
            'ApiURL'  => $RequestURL,
            'Content' => $EncodedContent,
            'Status'  => $Response->code,
            'Message' => $Response->content,
            'DecodedMessage' => $DecodedMessage,
        };

        # test if we have response codes to ignore
        if ($Response->code ~~ @{ $hParams->{'IgnoreCodes'} }) {
            Warning('API_CALL_ERROR', $ErrorObject);
            return;
        }

        Error('API_CALL_ERROR', $ErrorObject);
    }

    # return decoded content if we have a GET request only
    my $JSONResponse = $Response->decoded_content;
    return $JSONResponse ? decode_json($JSONResponse) : undef;

}

#========================================================================================
# §function     environment
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $store = $self->environment;
#----------------------------------------------------------------------------------------
# §description  Returns which Facebook Environment should be used.
#----------------------------------------------------------------------------------------
# §return       $environment | Facebook Environment | Object
#========================================================================================
sub environment {
    return shift->{'Environment'};
}

#========================================================================================
# §function     shopEnvironment
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Environment = $self->shopEnvironment($Shop);
#----------------------------------------------------------------------------------------
# §description  Returns the Facebook Shop Environment by the given shop.
#----------------------------------------------------------------------------------------
# §input        $Shop | shop object | Object
# §return       $Environment | Facebook Shop Environment | Object
#========================================================================================
sub shopEnvironment {
    my $self = shift;
    my ($Shop) = @_;

    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');    

    return $Shop->child('FacebookBusinessShopEnvironment') if $Shop->existsChild('FacebookBusinessShopEnvironment');
    return InsertObject('FacebookBusinessShopEnvironment', {
                                                            'Parent' => $Shop,
                                                            'Alias'  => 'FacebookBusinessShopEnvironment',
                                                           });
}

#========================================================================================
# §function     graphURL
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $GraphURL = $self->graphURL;
#----------------------------------------------------------------------------------------
# §description  Returns the facebook graph url for API requests.
#----------------------------------------------------------------------------------------
# §return       $GraphURL | Facebook Graph URL | string
#========================================================================================
sub graphURL {
    return FACEBOOK_GRAPH_URL;
}

#========================================================================================
# §function     apiVersion
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $GraphURL = $self->apiVersion;
#----------------------------------------------------------------------------------------
# §description  Returns the facebook API version.
#----------------------------------------------------------------------------------------
# §return       $GraphURL | Facebook API version | string
#========================================================================================
sub apiVersion {
    return FACEBOOK_API_VERSION;
}

#========================================================================================
# §function     appCredentials
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $hCredentials = $self->appCredentials;
#----------------------------------------------------------------------------------------
# §description  Returns which Facebook Environment should be used.
#----------------------------------------------------------------------------------------
# §return       $result | with following keys:
#               <ul>
#                   <li>AppID - facebook application id - string
#                   <li>AppSecret - facebook application secret - string
#               </ul> | ref.hash
#========================================================================================
sub appCredentials {
    return shift->environment->get([qw(AppID AppSecret)]);
}

#========================================================================================
# §function     logCommunication
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $RemoteFacebook->logCommunication( $Category, $HTTPObject );
# §example      $RemoteFacebook->logCommunication( 'FacebookBusiness::API::Object::RemoteFacebook', $Response );
#----------------------------------------------------------------------------------------
# §description  Logs HTTP communications.
#----------------------------------------------------------------------------------------
# §input        $Category | log category, undef for no logging | string
# §input        $Message | Message to be logged | HTTP::Request or HTTP::Response object
#========================================================================================
sub logCommunication {
    shift;
    my ($Category, $Message) = @_;

    my $Logger = GetCommunicationLog($Category);

    if (defined $Category && $Logger->is_debug) {
            if( $Message->isa('HTTP::Request') ) {
                $Logger->debug( "Request headers:\n" . $Message->headers_as_string );
                $Logger->debug( $Message->method .' '. $Message->uri ."\n". $Message->content);
            }
            else {
                $Logger->debug( "Response headers:\n" . $Message->headers_as_string );
                $Logger->debug( $Message->status_line . "\n" . $Message->content);
            }
    }
    return;
}

#========================================================================================
# §function     _getLogCategory
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $self->_getLogCategory();
#----------------------------------------------------------------------------------------
# §description  Returns the log category if logging is enabled.
#----------------------------------------------------------------------------------------
# §return       $LogCategory |  log category for enabled logging | string
#========================================================================================
sub _getLogCategory {
    my $self = shift;

    return undef unless $self->environment->get('FacebookBusinessEnableLogging');
    my $ShopID = $self->environment->getSite()->id;

    return  "FacebookBusiness::RemoteFacebook::$ShopID";
}

#========================================================================================
# §function     _buildRequestURL
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Shop->_buildRequestURL($APIUrl, $Route, $hQueryParams);
# §syntax       $Shop->_buildRequestURL($APIUrl, '/product_summary/search', {'gtin'=> '08154711'});
#----------------------------------------------------------------------------------------
# §description  Inserts the $Route into the given $APIUrl and adds $hQueryParams
#               afterwards to the URL.
#----------------------------------------------------------------------------------------
# §input        $APIUrl | API url | string
# §input        $Route | route to add to the url | string
# §input        $hQueryParams | locale id for call (optional) | string
# §return       $APIUrl |  api URL including the route and the query params | string
#========================================================================================
sub _buildRequestURL {
    my $self = shift;
    my ($APIUrl, $Route, $hQueryParams) = @_;

    my $ParamString = defined $hQueryParams ? BuildParameterString($hQueryParams) . '&' : '';
    $APIUrl .= "$Route?$ParamString";

    return $APIUrl;
}

1;
