#========================================================================================
# package      DE_EPAGES::MolliePayments::API::MolliePaymentsService
# state        public
# description  Service for Mollie
#========================================================================================
package DE_EPAGES::MolliePayments::API::MolliePaymentsService;

use strict;
use DE_EPAGES::EPagesJ::API::Object::RestClient;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

use constant BASE_URI =>  'payments/mollie/v2';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Mollie::API::MolliePaymentsService->new( %args );
#----------------------------------------------------------------------------------------
# description  Instantiates a new MolliePaymentsService with Epagesj Rest client set.
#----------------------------------------------------------------------------------------
# input        $RequestObject | request object to handle | object
# return       MolliePaymentsService | object | DE_EPAGES::MolliePayments::API::MolliePaymentsService
#========================================================================================
sub new {
    my $class = shift;
    my ($RequestObject)  = @_;

    my $self  = bless { 
        'request' => $RequestObject
    }, $class;

    return $self;
}

#========================================================================================
# function     withTestMode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Client = DE_EPAGES::Mollie::API::MolliePaymentsService->withTestMode();
#----------------------------------------------------------------------------------------
# description  Stores a variable in MolliePaymentsService object.
#----------------------------------------------------------------------------------------
# return       $MolliePaymentsService | MolliePaymentsService object | DE_EPAGES::MolliePaymentsService::API::MolliePaymentsService
#========================================================================================
sub withTestMode {
    my $self = shift;
    my ($IsTestMode) = @_;

    $self->{'IsInTestMode'} = $IsTestMode ? 1 : 0;

    return $self;
}

#========================================================================================
# function     doPostRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Response = $self->doPostRequest();
#----------------------------------------------------------------------------------------
# description  It calls (post) and retrieves the response information from epagesj
#               MolliePayments project and filters
#----------------------------------------------------------------------------------------
# return       $Response | filtered response hash | hash.ref
#========================================================================================
sub doPostRequest {
    my $self = shift;

    my $Request = $self->{'request'};
    my $RestClient = $self->_createEpagesjRestClient($Request->getShop());
    $self->{'client'} = $RestClient;

    my $RequestUrl = $self->_buildRequestUrl( $Request->getEpagesjRequestSuffixUrlPath('POST') );
    my $hRequestHeader = $Request->getHttpHeaders('POST');

    my $hRequestPayload = $Request->getPayload('POST');

    if ($self->{'IsInTestMode'}) {
        $hRequestPayload->{'testmode'} = 'true';
    }

    my $JsonRequestPayload = $RestClient->encodeRequest($hRequestPayload);

    my $Response = $RestClient->client()->post($RequestUrl, $hRequestHeader, $JsonRequestPayload);

    my $DecodedResponse = $self->_decodeFromResponse($Response);

    return $Request->handleResponse($DecodedResponse);
}

#========================================================================================
# function     doGetRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Response = $self->doGetRequest();
#----------------------------------------------------------------------------------------
# description  It calls (get) and retrieves the response information from epagesj
#               MolliePayments project and filters
#----------------------------------------------------------------------------------------
# return       $Response | filtered response hash | hash.ref
#========================================================================================
sub doGetRequest {
    my $self = shift;

    my $Request = $self->{'request'};
    my $RestClient = $self->_createEpagesjRestClient($Request->getShop());
    $self->{'client'} = $RestClient;

    my $Url = $self->_buildRequestUrl( $Request->getEpagesjRequestSuffixUrlPath('GET'));
    my $RequestUrl = URI::URL->new($Url);
    my $hQueryParams = $Request->getQueryParams();

    if ($self->{'IsInTestMode'}) {
        $hQueryParams->{'testmode'} = 'true';
    }

    $RequestUrl->query_form($hQueryParams);

    my $hRequestHeader = $Request->getHttpHeaders('GET');

    my $Response = $self->{'client'}->client()->get($RequestUrl, $hRequestHeader);

    my $DecodedResponse = $self->_decodeFromResponse($Response);

    return $Request->handleResponse($DecodedResponse);
}

#----------------------------------------------------------------------------------------
# Private methods
#----------------------------------------------------------------------------------------
sub _buildRequestUrl {
    my $self = shift;
    my ($RequestEndpoint) = @_; 

    my $RequestUrl = $self->{'client'}->{'BaseURL'};
    $RequestUrl .= BASE_URI;
    $RequestUrl .= $RequestEndpoint;

    return $RequestUrl;
}
#========================================================================================
sub _decodeFromResponse {
    my $self = shift;
    my ($Response) = @_;

    return $self->{'client'}->decodeResponse( $Response->decoded_content() );
}

#========================================================================================
sub _createEpagesjRestClient {
    my $self = shift;
    my ($Shop) = @_;

    my $System = LoadRootObject();
    my $hArgs = {};

    $hArgs->{BaseURL} = $System->get('OAuthServiceBaseUrl') // $System->get('EPagesJBaseURL');
    $hArgs->{ShopType} = $Shop->get('ShopTypeRef')->alias();
    $hArgs->{Shop} = $Shop;
    $hArgs->{BusinessUnit} = $System->get('ProviderID');

    return DE_EPAGES::EPagesJ::API::Object::RestClient->create( $System, $hArgs );
}

1;

