#========================================================================================
# package      DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerPaymentMethod
#----------------------------------------------------------------------------------------
# description  object interface for Payment.
#========================================================================================
package DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );

use DE_EPAGES::MolliePayments::API::Constants qw (
    INTERNAL_CARTRIDGE_NAME
    SUPPORTED_CURRENCIES
);

use strict;



#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethod->featureName;
#----------------------------------------------------------------------------------------
# description   Returns the feature name 'MolliePayments'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName {
     return INTERNAL_CARTRIDGE_NAME;
}



#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a payment method is visible in the storefront.
#               Overrides function <function DE_EPAGES::Payment::API::Object::PaymentMethod::usableAtStorefront>
#               to evaluate if payment method is configured
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $Usable | true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    my $hInfo = $self->get([qw( AccessToken RefreshToken ProfileId Method )]);

    my $IsUsableAtStorefront = 0;
    if( isValid($hInfo->{'AccessToken'}) &&
            isValid($hInfo->{'RefreshToken'}) &&
            isValid($hInfo->{'ProfileId'}) &&
            isValid($hInfo->{'Method'}) &&
            $self->SUPER::usableAtStorefront($hVars) ) {
        $IsUsableAtStorefront = 1;
    }

    return $IsUsableAtStorefront;
}



#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars)) {...
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               Valid currencies are: EUR, GBP
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       $canAddToBasket | true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    my $AreCurrenciesSupported = $hVars->{'CurrencyID'} ~~ SUPPORTED_CURRENCIES;
    my $CanBeAddedToBasket = $AreCurrenciesSupported && $self->SUPER::canAddToBasket($hVars);

    return $CanBeAddedToBasket;
}



#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsInTestMode' => 1
    };
}

1;