#========================================================================================
# package      DE_EPAGES::MolliePayments::API::Requests::Tokens
# state        public
# description  Helper module for generating MolliePayments tokens requests to epagesj
#========================================================================================
package DE_EPAGES::MolliePayments::API::Requests::Tokens;

use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

use URI;
use URI::QueryParam;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::MolliePayments::API::Requests::Tokens->new( );
#----------------------------------------------------------------------------------------
# description  MolliePayments tokens requests to epagesj helper constructor
#----------------------------------------------------------------------------------------
# input        $hTokensData | required data to generate tokens requests to epagesj | hash.ref
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::Tokens
#========================================================================================
sub new {
    my $class = shift;
    my ($hTokensData) = @_;

    TestObject(
            'PaymentMethodMolliePayments',
            $hTokensData->{'payment'},
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments'
    );

    my $self = bless {
        'data' => $hTokensData,
        'payload' => {}
    }, $class;

    return $self;
}

#========================================================================================
# function     withAuthorizationCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->withAuthorizationCode( );
#----------------------------------------------------------------------------------------
# description  Mark authorization code as part of the payload
#----------------------------------------------------------------------------------------
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::Tokens
#========================================================================================
sub withAuthorizationCode {
    my $self = shift;

    $self->{'payload'}->{'code'} = $self->{'data'}->{'code'};

    return $self;
}

#========================================================================================
# function     withRefreshToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->withRefreshToken( $RefreshToken );
#----------------------------------------------------------------------------------------
# description  Set refresh token into token object's payload
#----------------------------------------------------------------------------------------
# return       $Object | new object | DE_EPAGES::MolliePayments::API::Requests::Tokens
#========================================================================================
sub withRefreshToken {
    my $self = shift;

    $self->{'payload'}->{'refreshToken'} = $self->{'data'}->{'payment'}->get('RefreshToken');

    return $self;
}

#========================================================================================
# function     getEpagesjRequestSuffixUrlPath
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Url = $TokensRequest->getEpagesjRequestSuffixUrlPath( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the tokens postfix url path from the epagesj endpoint
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $Url | tokens suffix url path from the epagesj endpoint | string
#========================================================================================
sub getEpagesjRequestSuffixUrlPath {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq 'POST') {
        return '/tokens';
    }
}

#========================================================================================
# function     getHttpHeaders
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hRequestHeader = $self->getHttpHeaders( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns required headers of the tokens request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hRequestHeader | tokens request headers hash | hash.ref
#========================================================================================
sub getHttpHeaders {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq 'POST') {
        return {};
    }
}

#========================================================================================
# function     getPayload
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hPayload = $self->getPayload( $HttpRequestMethod );
#----------------------------------------------------------------------------------------
# description  It returns the body payload of the tokens request
#----------------------------------------------------------------------------------------
# input        $HttpRequestMethod | HTTP request method string | string
# return       $hPayload | tokens request payload hash | hash.ref
#========================================================================================
sub getPayload {
    my $self = shift;
    my ($HttpRequestMethod) = @_;

    if ($HttpRequestMethod eq 'POST') {
        my $System = LoadRootObject();
        $self->{'payload'}->{'clientId'} = $System->get('MolliePayments_AppClientID');
        $self->{'payload'}->{'clientSecret'} = $System->get('MolliePayments_AppClientSecret');
        $self->{'payload'}->{'redirectUri'} = $System->get('MolliePayments_AppRedirectURL');

    }

    return $self->{'payload'};
}

#========================================================================================
# function     getShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Shop = $self->getShop();
#----------------------------------------------------------------------------------------
# description  It returns the shop object from state URL argument
#----------------------------------------------------------------------------------------
# return       $Shop | shop object | object
#========================================================================================
sub getShop {
    my $self = shift;

    return $self->{'data'}->{'payment'}->getSite();
}

#========================================================================================
# function     handleResponse
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hFilteredResponse = $self->handleResponse($hResponse);
#----------------------------------------------------------------------------------------
# description  It returns the response filtered
#----------------------------------------------------------------------------------------
# input        $hResponse | response hash | hash.ref
# return       $hFilteredResponse | filtered response hash | hash.ref
#========================================================================================
sub handleResponse {
    my $self = shift;
    my ($hResponse) = @_;

    return $hResponse;
}

1;

