#========================================================================================
# package      DE_EPAGES::MolliePayments::UI::JSON::BasketOrder
# base         DE_EPAGES::Presentation::UI::JSON
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for basket or order
#========================================================================================
package DE_EPAGES::MolliePayments::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use DE_EPAGES::MolliePayments::API::MolliePaymentsService;
use DE_EPAGES::MolliePayments::API::Requests::Methods;
use DE_EPAGES::MolliePayments::API::Requests::Profiles;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::MolliePayments::API::Constants qw ( PAYMENT_TYPE_MOLLIE_PAYMENTS );

use feature 'switch';

use strict;

#========================================================================================
# function     GetMolliePaymentsProfiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetMolliePaymentsProfiles( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for GetMolliePaymentsProfiles at MBO ViewSettings
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub GetMolliePaymentsProfiles {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $IsInTestMode = $Servlet->params()->{'IsInTestMode'};

    my $ProfileHelper = $self->_createProfilesHandler({
        'payment' => $PaymentMethod
    });
    my $ProfileId = $Servlet->params()->{'ProfileId'};
    $ProfileHelper->withFrom($ProfileId);
    
    my $MollieService = $self->_createMolliePaymentsService($ProfileHelper);

    my $hProfilesResponse = {};

    eval {
        $hProfilesResponse = $MollieService->withTestMode($IsInTestMode)->doGetRequest();
    };

    if (ExistsError()) {
        my $Error = GetError();
        $self->_LogPayment(
            'Error when getting profiles', $Error->shortMessage(), undef, undef
        );
        $Servlet->vars('MollieConnectionError', 1);
        return $self->_writeContentOutput($hProfilesResponse);
    }

    $self->_LogPayment(
        'Get Profiles',
        { 'PaymentMethod' =>  $PaymentMethod->id(), 'Profiles Response' => $hProfilesResponse },undef , undef
    );

    return $self->_writeContentOutput($hProfilesResponse);
}

#========================================================================================
# function     GetMolliePaymentsMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetMolliePaymentsMethods( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller for GetMolliePaymentsMethods at MBO ViewSettings
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub GetMolliePaymentsMethods {
    my $self = shift;
    my ($Servlet) = @_;

    my $IsInTestMode = $Servlet->params()->{'IsInTestMode'};
    my $ProfileId = $Servlet->params()->{'ProfileId'};

    my $PaymentMethod = $Servlet->object();
    my $hMethodsData = {
        'payment' => $PaymentMethod
    };
    my $MethodsHandler = $self->_createMethodsHandler($hMethodsData);
    $MethodsHandler->withProfileId($ProfileId);
    my $MollieService = $self->_createMolliePaymentsService($MethodsHandler);

    my $hLogParams = {
        'PaymentMethodId' =>  $PaymentMethod->id(),
        'profileId' => $ProfileId,
    };
    my $hMethodsResponse = {};

    eval {
        $hMethodsResponse = $MollieService->withTestMode($IsInTestMode)->doGetRequest();
    };
    if (ExistsError()) {
        my $Error = GetError();
        $hLogParams->{'Error'} = $Error->shortMessage();
        $self->_LogPayment('Error when getting Payment Methods',$hLogParams, undef, undef);
        $Servlet->vars('MollieConnectionError', 1);
        return $self->_writeContentOutput($hMethodsResponse);
    }

    $hLogParams->{'MethodsResponse'} = $hMethodsResponse;
    $self->_LogPayment('Get Payment Methods response', $hLogParams, undef , undef);

    return $self->_writeContentOutput($hMethodsResponse);
}

#----------------------------------------------------------------------------------------
sub _createProfilesHandler {
    my $self = shift;
    my ($hData) = @_;

    return DE_EPAGES::MolliePayments::API::Requests::Profiles->new($hData);
}

#----------------------------------------------------------------------------------------
sub _createMethodsHandler {
    my $self = shift;
    my ($hMethodsData) = @_;

    return DE_EPAGES::MolliePayments::API::Requests::Methods->new($hMethodsData);
}
#----------------------------------------------------------------------------------------
sub _createMolliePaymentsService {
    my $self = shift;
    my ($HandlerObject) = @_;

    return DE_EPAGES::MolliePayments::API::MolliePaymentsService->new($HandlerObject);
}

#----------------------------------------------------------------------------------------
sub _writeContentOutput {
    my $self = shift;
    my ($hData) = @_;

    $self->writeContentOutput($hData);
    return;
}

#----------------------------------------------------------------------------------------
sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_MOLLIE_PAYMENTS, $Name, $DebugData, $Section, $Filter);
}

1;
