#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::MolliePaymentsService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::MolliePayments::API::MolliePaymentsService
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::MolliePaymentsService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use JSON::Any;
use Test::Exception;
use Test::Mockify::Matcher qw ( Object String );
use Test::Mockify::Sut;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant ACCESS_TOKEN => 'FAKE_ACCESS_TOKEN';

use constant EPAGESJ_SUFFIX_URL_PATH => '/epjsuffix/path';

use constant DECODED_JSON_RESPONSE_AS_HASH => { 'resfield1' => 'resvalue1', 'resfield2' => 'resvalue2' };

sub test {
    my $self = shift;

    $self->test_MolliePaymentsServiceTokensWithPost();
    $self->test_MolliePaymentsServiceMethodsWithGet();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_MolliePaymentsServiceTokensWithPost {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT2($self->_createRequestTokensHelper());
    my $Response = $SUT->doPostRequest();

    ok(WasCalled(
            $SUT, 'DE_EPAGES::EPagesJ::API::Object::RestClient::create'),
            "$SubTestName - epagesj client set in the service."
    );

    is_deeply(
            GetParametersFromMockifyCall($SUT, '_buildRequestUrl'),
            [EPAGESJ_SUFFIX_URL_PATH],
            "$SubTestName - request URL is built and set in the service."
    );

    is_deeply($Response, DECODED_JSON_RESPONSE_AS_HASH, "$SubTestName - Service response validated.");

    ok(WasCalled($SUT->{'client'}->client(), 'post'), "$SubTestName - post http request is performed by the service.");

    ok(WasCalled($SUT, '_decodeFromResponse'), "$SubTestName - http response is gotten and processed by the service.");
}

sub test_MolliePaymentsServiceMethodsWithGet {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $RequestHelper = $self->_createMethodsHelper();
    $RequestHelper->withProfileId("fake_profile_id");

    my $SUT = $self->_createSUT2($RequestHelper);
    my $Response = $SUT->withTestMode(1)->doGetRequest();

    ok(WasCalled(
            $SUT, 'DE_EPAGES::EPagesJ::API::Object::RestClient::create'),
            "$SubTestName - epagesj client set in the service."
    );

    is_deeply(
            GetParametersFromMockifyCall($SUT, '_buildRequestUrl'),
            [EPAGESJ_SUFFIX_URL_PATH],
            "$SubTestName - request URL is built and set in the service."
    );

    is_deeply($Response, DECODED_JSON_RESPONSE_AS_HASH, "$SubTestName - Service response validated.");

    my $ExpectedUrl = URI::URL->new("payments/mollie/v2/epjsuffix/path?profileId=fake_profile_id&testmode=true&locale=de_DE");

    is_deeply(
            GetParametersFromMockifyCall($SUT->{'client'}->client(), 'get'),
            [$ExpectedUrl, {'Authorization' => 'Basic f3adfadsraradgfgh'}],
            "$SubTestName - GET query params are correctly built and set in the service."
    );

    ok(WasCalled($SUT->{'client'}->client(), 'get'), "$SubTestName - get http request is performed by the service.");

    ok(WasCalled($SUT, '_decodeFromResponse'), "$SubTestName - http response is gotten and processed by the service.");
}
#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _createPaymentMethod {
    my $self = shift;
    my ($hVars) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', {}
    );
    $Mockify->addMockWithReturnValue('getSite', $self->_createShop());

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'MyShop',
        'ShopTypeRef' => $self->_createShopTypeRef()
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );
    return $Mockify->getMockObject();
}

sub _createSUT2 {
    my $self = shift;
    my ($RequestHelper) = @_;

    my $MockifySut = Test::Mockify::Sut->new(
            'DE_EPAGES::MolliePayments::API::MolliePaymentsService', [ $RequestHelper ]
    );

    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
            ->whenAny()
            ->thenReturn($self->_createSystem());

    $MockifySut->mockStatic('DE_EPAGES::EPagesJ::API::Object::RestClient::create')
            ->whenAny()
            ->thenReturn($self->_createRestClient());

    $MockifySut->spy('_buildRequestUrl')
            ->when(String());

    $MockifySut->spy('_decodeFromResponse')
            ->when(Object('HTTP::Response'));

    return $MockifySut->getMockObject();
}

sub _createRequestTokensHelper {
    my $self = shift;

    my $aParameters = [
        {
            'payment' => $self->_createPaymentMethod(),
            'code' => 'MyAuthorizationCode'
        }
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::MolliePayments::API::Requests::Tokens', $aParameters);

    $Mockify->addMockWithReturnValue('getShop', $self->_createShop());
    $Mockify->addMockWithReturnValueAndParameterCheck(
            'getEpagesjRequestSuffixUrlPath', EPAGESJ_SUFFIX_URL_PATH, ['string']
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
            'getHttpHeaders', {'Authorization' => 'Basic f3adfadsraradgfgh'} , ['string']
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
                'getPayload',
                {'reqfield1' => 'reqvalue1', 'reqfield2' => 'reqvalue2', 'reqfield3' => 'reqvalue3'} ,
                ['string']
    );
    return $Mockify->getMockObject();
}

sub _createMethodsHelper {
    my $self = shift;

    my $aParameters = [
        {
            'payment' => $self->_createPaymentMethod(),
        }
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::MolliePayments::API::Requests::Methods', $aParameters);

    $Mockify->addMockWithReturnValue('getShop', $self->_createShop());
    $Mockify->addMockWithReturnValueAndParameterCheck(
            'getEpagesjRequestSuffixUrlPath', EPAGESJ_SUFFIX_URL_PATH, ['string']
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
            'getHttpHeaders', {'Authorization' => 'Basic f3adfadsraradgfgh'} , ['string']
    );
    $Mockify->addMockWithReturnValue('_getMerchantLocaleId', "de_DE");

    return $Mockify->getMockObject();
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'EPagesJBaseURL' => 'https://shop.example.com:8088/rs',
        'OAuthServiceBaseUrl' => undef,
        'ProviderID' => 'epages'
    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );

    return $MockifySystem->getMockObject();
}

sub _createRestClient {
    my $self = shift;

    my $aParameters = [];
    my $MockifyRestClient = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::EPagesJ::API::Object::RestClient', $aParameters
    );
    $MockifyRestClient->addMockWithReturnValue('client', $self->_createHttpClient());

    return $MockifyRestClient->getMockObject();
}

sub _createHttpClient {
    my $self = shift;

    my $MockifyHttpClient = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::EPagesJ::API::Object::HTTPClient');

    $MockifyHttpClient->addMockWithReturnValueAndParameterCheck(
            'get', $self->_createHttpResponse, ['object', 'hashref']
    );
    $MockifyHttpClient->addMockWithReturnValueAndParameterCheck(
            'post', $self->_createHttpResponse, ['string', 'hashref', 'any']
    );
    $MockifyHttpClient->addMockWithReturnValueAndParameterCheck(
            'put', $self->_createHttpResponse, ['string', 'hashref', 'any']
    );

    return $MockifyHttpClient->getMockObject();
}

sub _createHttpResponse {
    my $self = shift;

    my $Json = JSON::Any->new();

    my $MockifyHttpResponse = DE_EPAGES::Test::Mock::Mockify->new('HTTP::Response');
    $MockifyHttpResponse->addMockWithReturnValue('decoded_content', $Json->objToJson(DECODED_JSON_RESPONSE_AS_HASH));

    return $MockifyHttpResponse->getMockObject();
}

sub _createShopTypeRef {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'UnityDemo'
    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::ShopTypeRef', $hAttributes
    );

    return $MockifySystem->getMockObject();
}


#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
