#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::Requests_Methods
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::MolliePayments::API::Requests::Methods
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::Requests_Methods;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;

use Test::Mockify::Sut;
use Test::Exception;
use Test::More;

use strict;

use constant MERCHANT_LOCALE => 'de_DE';
use constant ACCESS_TOKEN => 'MyAccessToken';
use constant REFRESH_TOKEN => 'MyRefreshtoken';
use constant PROFILE_ID => 'pfl_v9hTwCvYq1';
use constant Methods_SUFFIX_EPAGESJ_PATH => '/methods';
use constant CLIENT_ID => 'MyClientID';
use constant CLIENT_SECRET => 'MyClientSecret';
use constant CLIENT_REDIRECT_URI =>
        'https://myshop.epages.de/epages/Site.admin/?ChangeAction=BacklinkMolliePaymentsOnboarding';

use constant RESPONSE_HASH => {
  'methods' => [
    {
      'description' => 'iDEAL',
      'id' => 'ideal'
    }
  ]
};

use constant HTTP_HEADERS_HASH => {
        'X-AccessToken' => ACCESS_TOKEN
};

sub test {
    my $self = shift;

    $self->test_Methods();
    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------

sub test_Methods {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT;
    lives_ok(sub { $SUT = $self->_createSUT(); }, "$SubTestName - Methods is instantiated ok");

    is(
            $SUT->getEpagesjRequestSuffixUrlPath('GET'),
            Methods_SUFFIX_EPAGESJ_PATH,
            "$SubTestName - Methods object provides epagesj suffix path."
    );

    my $hActualQueryParams = $SUT->withProfileId(PROFILE_ID)->getQueryParams();
    my $hExpectedQueryParams = {
        'profileId' => PROFILE_ID,
        'locale' => MERCHANT_LOCALE
    };

    is_deeply( $hActualQueryParams, $hExpectedQueryParams, "$SubTestName - Methods gives correct query params.");

    is_deeply( $SUT->getHttpHeaders('GET'), HTTP_HEADERS_HASH, "$SubTestName - Methods object provides HTTP headers.");
}

sub test_Methods_provision_for_shop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Shop = $SUT->getShop();

    is( $Shop->alias(), 'MyShop', "$SubTestName - Methods object provides shop object.");
}

sub test_Methods_handling_for_response {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();

    is_deeply(
            $SUT->handleResponse(RESPONSE_HASH),
            RESPONSE_HASH,
            "$SubTestName - Methods response filtering provides original response."
    );
}


#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $hData = $self->_generateRequestData();
    my $aParameterList = [$hData];
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::MolliePayments::API::Requests::Methods', $aParameterList);

    $MockifySut->mock('_getMerchantLocaleId')->when()->thenReturn(MERCHANT_LOCALE);

    return $MockifySut->getMockObject();
}

sub _generateRequestData {
    my $self = shift;

    return {
        'payment' => $self->_createPaymentMethod()
    };
}

sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'AccessToken' => ACCESS_TOKEN,
        'RefreshToken' => REFRESH_TOKEN,
        'ProfileId' => PROFILE_ID,
        'Site' => $self->_createShop()
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'MyShop'
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
