#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::Requests_Tokens
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::MolliePayments::API::Requests::Tokens
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::Requests_Tokens;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;

use Test::Mockify::Sut;
use Test::More;

use strict;

use constant AUTHORIZATION_CODE => 'MyAuthorizationCode';

use constant REFRESH_TOKEN => 'MyRefreshtoken';

use constant PAYMENT_OBJECT_ID => 23101;

use constant TOKENS_SUFFIX_EPAGESJ_PATH => '/tokens';

use constant CLIENT_ID => 'MyClientID';

use constant CLIENT_SECRET => 'MyClientSecret';

use constant CLIENT_REDIRECT_URI =>
        'https://myshop.epages.de/epages/Site.admin/?ChangeAction=BacklinkMolliePaymentsOnboarding';

use constant RESPONSE_HASH => {
    'access_token' => 'MyAccessToken',
    'refresh_token' => REFRESH_TOKEN,
    'expires_in' => 3600,
    'token_type' => 'bearer',
    'scope' => 'payments.read payments.write refunds.read refunds.write profiles.read profiles.write'
};


sub test {
    my $self = shift;

    $self->test_Tokens_creation();
    $self->test_Tokens_provision_for_post_path_and_headers();
    $self->test_Tokens_provision_for_post_payload_with_code();
    $self->test_Tokens_provision_for_post_payload_with_refresh_token();
    $self->test_Tokens_provision_for_shop();
    $self->test_Tokens_handling_for_response();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_Tokens_creation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    is( scalar(keys %{$SUT->{'data'}}), 2, "$SubTestName - Tokens object gets initialized with some arguments.");

    is( scalar(keys %{$SUT->{'payload'}}), 0, "$SubTestName - Tokens object gets initialized with an empty payload.");
}

sub test_Tokens_provision_for_post_path_and_headers {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    is(
            $SUT->getEpagesjRequestSuffixUrlPath('POST'),
            TOKENS_SUFFIX_EPAGESJ_PATH,
            "$SubTestName - Tokens object provides epagesj suffix path."
    );

    is_deeply( $SUT->getHttpHeaders('POST'), {}, "$SubTestName - Tokens object provides HTTP headers.");
}

sub test_Tokens_provision_for_post_payload_with_code {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Payload = $SUT
            ->withAuthorizationCode()
            ->getPayload('POST');

    is( $Payload->{'clientId'}, CLIENT_ID, "$SubTestName - Tokens payload client id is retrieved.");

    is( $Payload->{'clientSecret'}, CLIENT_SECRET, "$SubTestName - Tokens payload client secret is retrieved.");

    is( $Payload->{'redirectUri'}, CLIENT_REDIRECT_URI, "$SubTestName - Tokens payload redirect URI is retrieved.");

    is( $Payload->{'code'}, AUTHORIZATION_CODE, "$SubTestName - Tokens payload authorization code is retrieved.");

    is( $Payload->{'refreshToken'}, undef, "$SubTestName - Tokens payload refresh token is NOT retrieved.");
}

sub test_Tokens_provision_for_post_payload_with_refresh_token {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Payload = $SUT
            ->withRefreshToken()
            ->getPayload('POST');

    is( $Payload->{'refreshToken'}, REFRESH_TOKEN, "$SubTestName - Tokens payload refresh token is retrieved.");

    is( $Payload->{'code'}, undef, "$SubTestName - Tokens payload authorization code is NOT retrieved.");
}

sub test_Tokens_provision_for_shop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Shop = $SUT->getShop();

    is( $Shop->alias(), 'MyShop', "$SubTestName - Tokens object provides shop object.");
}

sub test_Tokens_handling_for_response {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();

    is_deeply(
            $SUT->handleResponse(RESPONSE_HASH),
            RESPONSE_HASH,
            "$SubTestName - Tokens response filtering provides original response."
    );
}


#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

        my $hData = $self->_generateRequestData();
    my $aParameterList = [$hData];
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::MolliePayments::API::Requests::Tokens', $aParameterList);

    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
            ->whenAny()
            ->thenReturn($self->_createSystem());

    return $MockifySut->getMockObject();
}

sub _generateRequestData {
    my $self = shift;

    return {
        'payment' => $self->_createPaymentMethod(),
        'code' => AUTHORIZATION_CODE
    };
}

sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'RefreshToken' => REFRESH_TOKEN,
        'Site' => $self->_createShop()
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'MolliePayments_AppClientID' => CLIENT_ID,
        'MolliePayments_AppClientSecret' => CLIENT_SECRET,
        'MolliePayments_AppRedirectURL' => CLIENT_REDIRECT_URI
    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );

    return $MockifySystem->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'MyShop'
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
