#==============================================================================
# package      DE_EPAGES::Ogone::UI::PaymentMethodOgone
# state        private
#==============================================================================
package DE_EPAGES::Ogone::UI::PaymentMethodOgone;
use base DE_EPAGES::ExternalPayment::UI::PaymentMethodExternalPayment;

use strict;

use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::Object::API::Factory qw ( Transaction LoadObjectByPath JoinObjectPath );
use DE_EPAGES::Payment::API::Table::ShippingPaymentMap qw ( InsertShippingPaymentMap );
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Payment::API::Shop qw (GetShopPaymentLogos);
use DE_EPAGES::Payment::API::Table::ShopPaymentLogo qw( InsertShopPaymentLogo ExistsShopPaymentLogo );
use DE_EPAGES::Ogone::API::Constants qw (@SPLIT_GATEWAY_METHODS $SPLIT_GATEWAY_METHODS_MAXROWS);

#==============================================================================

sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;
    $Servlet->vars('UpdateURL', BuildShopUrl($Servlet->object->getSite, {'ChangeAction' => 'PaymentCallbackOgone' },
                                                {'UseSSL' => 1, 'UseObjectPath'=> 1}));

    return;
}

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::View($Servlet);

    my $PaymentMethod = $Servlet->object();
    my $PaymentMethodClassName = $PaymentMethod->class()->alias();

    $self->ViewSettingsMultiPayment($Servlet, $PaymentMethodClassName, 'SplitGatewayMethod');

    return;
}

sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $Form = $Servlet->form();
    my $Shop = $PaymentMethod->getSite();
    my $Folder = $Shop->folder('PaymentMethods');

    # In order to show the sub payment method in the general page of the payment methods
    # we rewrite the Name and Alias before we save the payment method
    my $SplitGatewayMethod = $Form->{'Values'}->{'SplitGatewayMethod'}[0];

    my %hSPLIT_GATEWAY_METHODS = map { ($_->{'ID'}) => $_ } @SPLIT_GATEWAY_METHODS;
    my $nameMethod = $hSPLIT_GATEWAY_METHODS{$SplitGatewayMethod}->{'UIName'} //
                     $hSPLIT_GATEWAY_METHODS{$SplitGatewayMethod}->{'Name'};
    my $aliasMethod = $nameMethod;


    #Check if the payment method for the same payment type already exists, before rewriting
    if ( $Folder->existsChild($aliasMethod)  and
         $PaymentMethod->get('Alias') ne $aliasMethod ) {

            $Form->addFormError({
                'Form'  => 'PaymentMethodOgone',
                'Reason'=> 'OBJECT_EXISTS',
                'Name'  => $SplitGatewayMethod
            });
            $Form->executeFormError;
        }

    # Rewrite the name and the alias of the payment method
    $PaymentMethod->set({'Name' => $nameMethod});
    $PaymentMethod->set({'Alias' => $aliasMethod});

    # some form checks here
    if ( $Form->{'Values'}->{'IsInTestMode'}[0] ) {
        $Form->form($PaymentMethod, 'OgoneTestSettings');
    } else {
        $Form->form($PaymentMethod, 'OgoneSettings');
    }

    #--- standard save
    $self->SUPER::Save($Servlet);

    return;
}

#========================================================================================
# function     GetSplitGatewayMethods
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSplitGatewayMethods = GetSplitGatewayMethods( );
#----------------------------------------------------------------------------------------
# description  It provides the multi-payment structure of grouped payment methods.
#----------------------------------------------------------------------------------------
# return       $aSplitGatewayMethods | hash with split gateway methods | ref.array
#========================================================================================
sub GetSplitGatewayMethods {
    my $self = shift;

    return \@SPLIT_GATEWAY_METHODS;
}

#========================================================================================
# function     GetSplitGatewayMethodsMaxRow
# state        public
#----------------------------------------------------------------------------------------
# syntax       $MaxRow = GetSplitGatewayMethodsMaxRow( );
#----------------------------------------------------------------------------------------
# description  It provides the maximum number of payment methods of any group.
#----------------------------------------------------------------------------------------
# return       $MaxRow | maximum number of payment methods per group| integer
#========================================================================================
sub GetSplitGatewayMethodsMaxRow {
    my $self = shift;

    return $SPLIT_GATEWAY_METHODS_MAXROWS;
}

#========================================================================================
# function     NewMultiPaymentMethodOgone
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->NewMultiPaymentMethodOgone( $Servlet );
#----------------------------------------------------------------------------------------
# description  Payment method replication with a different method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub NewMultiPaymentMethodOgone {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::NewMultiPaymentMethod($Servlet);

    return ;
}

#==============================================================================
# function     NewPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->NewPaymentMethod( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName );
#----------------------------------------------------------------------------------------
# description  It creates a new payment method.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
# input        $PaymentMethod | payment method object | object
# input        $SplitGatewayMethod | payment method object | object
# input        $MethodName | method name | string
# input        $LogoName | logo name | string
# return       $NewPaymentMethod | payment method | object
#========================================================================================
sub NewPaymentMethod {
    my $self = shift;

    my ( $Servlet, $PaymentMethod, $SplitGatewayMethod, $MethodName, $LogoName ) = @_;

    my $Shop = $PaymentMethod->getSite();
    my $MethodAlias = $PaymentMethod->get('PaymentType')->get('Alias'); # it only can be Ogone or Postfinance
    my $PaymentType = LoadObjectByPath(JoinObjectPath(['PaymentTypes', $MethodAlias]));

    return unless defined $SplitGatewayMethod && $SplitGatewayMethod =~ /\S/;

    my $aLogos = GetShopPaymentLogos($Shop, LoadObjectByPath(JoinObjectPath(['PaymentTypes', $MethodAlias])));
    #my $SplitGatewayMethodAlias;
    my $PaymentTypeLogoID;
    foreach my $hLogo (@$aLogos) {
        if ($LogoName eq $hLogo->{'PaymentLogo'}->alias) {
            $PaymentTypeLogoID = $hLogo->{'PaymentTypeLogo'}->id;
        }
    }

    eval {
        Transaction( sub {
            $PaymentMethod = $PaymentType->newPaymentMethod($Shop, {
                'MerchantNameTest' => $PaymentMethod->get('MerchantNameTest'),
                'MerchantPasswordTest' => $PaymentMethod->get('MerchantPasswordTest'),
                'PassphraseTest' => $PaymentMethod->get('PassphraseTest'),
                'MerchantName' => $PaymentMethod->get('MerchantName'),
                'MerchantPassword' => $PaymentMethod->get('MerchantPassword'),
                'Passphrase' => $PaymentMethod->get('Passphrase'),
                'SplitGatewayMethod' => $SplitGatewayMethod,
                'Alias' => $MethodName,
                'Position' => 9999,
                'Shop' => $Shop,
            });

            # activate on all shipping methods
            InsertShippingPaymentMap({
                'ShippingMethodID'  => $_->id,
                'PaymentMethodID'   => $PaymentMethod->id
            }) foreach @{$Shop->get('ShippingMethods')};

            #insert new Logos
            if ($PaymentTypeLogoID){
                InsertShopPaymentLogo({
                    'ShopID'            => $Shop->id,
                    'PaymentTypeLogoID' => $PaymentTypeLogoID,
                }) unless ExistsShopPaymentLogo({
                    'ShopID'            => $Shop->id,
                    'PaymentTypeLogoID' => $PaymentTypeLogoID,
                });
            }
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'FeatureLimitExceededGeneral') {
            $Servlet->vars('FeatureMaxValueReachedGeneral',1);
        }
        elsif ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
        }
        else {
            Error($Error);
        }
    }

    return $PaymentMethod;
}

1;
