use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Order::API::Cartridge;
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         FixNewOrderNotification
# state            public
#----------------------------------------------------------------------------------------
# syntax           FixNewOrderNotification();
#----------------------------------------------------------------------------------------
# description      Fixes the new order notification for a given shop
#========================================================================================
sub FixNewOrderNotification {
    my ($ShopAlias) = @_;

    my $Shop = LoadObjectByPath('/Shops/'.$ShopAlias);

    if ($Shop->isUnity()) {
        my $ShopNotifications = $Shop->folder('ShopNotifications');
        my $Cartridge = DE_EPAGES::Order::API::Cartridge->new;

        ImportObjects($Cartridge->xmlDirectory().'/ShopNotifications.xml', $Shop->pathString)
            unless $ShopNotifications->existsChild('NewOrders');
        my $NewOrdersNotification = $ShopNotifications->child('NewOrders');
        $NewOrdersNotification->deleteAllObjects();
        my $aOrders = $Shop->get('Orders');

        foreach my $Order (@$aOrders) {
            my $ViewedOn = $Order->get('ViewedOn');
            # order unseen
            if (!defined $ViewedOn) {
                $NewOrdersNotification->addObject($Order);
            }
        }
    } else {
        # non Unity shop
        print 'Shop ' . $ShopAlias . " is not a Unity shop.\n";
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ShopAlias;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shopalias=s' => \$ShopAlias,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }

    unless ( defined $ShopAlias ) {
        print STDERR "Missing parameter -shopalias.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { FixNewOrderNotification($ShopAlias); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Fixes the new order notification for a given shop

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
    -shopalias  alias of a single shop                          ()

flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
