#========================================================================================
# package      DE_EPAGES::RemoteSearch::Hooks::Shop
# state        private
#----------------------------------------------------------------------------------------
# description  Initially sets up search index for a shop
#========================================================================================
package DE_EPAGES::RemoteSearch::Hooks::Shop;

use strict;

use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::RemoteSearch::API::RemoteSearchImpl qw (
    DropAllSearchIndexes
    GetOrCreateSearchIndex
);

#========================================================================================
# function     OnInsertShop
#----------------------------------------------------------------------------------------
# syntax       OnInsertShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Create search index on shop creation. Initial index status is "Unitialized"
#               and needs to be initialized by manageIndex.pl script.
#----------------------------------------------------------------------------------------
# input        $hParams |
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    if (LoadRootObject->get('RemoteSearchEnabled')) {
        GetOrCreateSearchIndex($Shop)->initialize();
    }
    return;
}

#========================================================================================
# function     OnDeleteShop
#----------------------------------------------------------------------------------------
# syntax       OnDeleteShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Deletes the search index to avoid deleting every single product from the
#               index.
#----------------------------------------------------------------------------------------
# input        $hParams |
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    DropAllSearchIndexes( $Shop, 1 );
    return;
}

#========================================================================================
# function     OnUpdateShop
#----------------------------------------------------------------------------------------
# syntax       OnUpdateShop( $hInfo );
#----------------------------------------------------------------------------------------
# description  Updates the search index, if search-relevant changes were made to the
#               Shop. (actually REST-API caches some shop infos, so it is not search-
#               relevant, but REST API relevant changes. This is going to be resolved
#               in the future)
#----------------------------------------------------------------------------------------
# input        $hInfo |
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnUpdateShop {
    my ($hInfo) = @_;
    my $Shop = $hInfo->{Object};
    my $hValues = $hInfo->{'Values'};

    # Lists of Attributes, which trigger update
    my @AttributesTriggerReindex = qw ( 
        Alias WebServerScriptNamePart FacetedSearchShowFacets TaxModel 
        UnityFacetedSearchFilterCategory UnityFacetedSearchFilterManufacturer 
        UnityFacetedSearchFilterPriceRange 
        DomainName HasSSLCertificate
    );
    
    if (@AttributesTriggerReindex ~~ %$hValues) {
        GetOrCreateSearchIndex($Shop)->initialize();
    }
    return;
}

1;
