#========================================================================================
# package      DE_EPAGES::ShopConfiguration::API::Object::Provider
# state        public
# base         DE_EPAGES::Object::API::Object::Site
# description  object interface for Provider.
#========================================================================================
package DE_EPAGES::ShopConfiguration::API::Object::Provider;
use base DE_EPAGES::Object::API::Object::Site;

use strict;
use feature 'switch';

use DE_EPAGES::Core::API::Error qw ( Error GetError ExistsError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Core::API::PerlTools  qw( ElementOf GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadRootObject LoadClassByAlias JoinObjectPath );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Search::API::Object::PagerSearch;
use DE_EPAGES::Search::API::SearchDefinition qw( GetSearchDefinition );
use DE_EPAGES::ShopConfiguration::API::Table::ProviderStore qw(
    DeleteProviderStore
    ExistsProviderStore
    GetInfoProviderStore
    GetPKeyProviderStoresByProviderID
    InsertProviderStore
);
##no critic (RequireCorrectImports)
use DE_EPAGES::ShopConfiguration::API::XML::ProviderImportHandler;
use DE_EPAGES::ShopConfiguration::API::XML::ProviderDeleteHandler;
use DE_EPAGES::ShopConfiguration::API::XML::ProviderExportDriver;
##use critic
use DE_EPAGES::ShopConfiguration::API::PreCreatedShop qw( GetPreCreatedShop );
use DE_EPAGES::XML::API::SOAPDateTimeFormatXML;

sub ErrorDefinition {
    return {
        'ShopRefAlias_Exists' => 'The shop alias #ShopRefAlias already exists for provider #ProviderAlias',
        'ShopRefAlias_ContainsIllegalCharacters' => 'The WebServerScriptNamePart #ShopRefWebServerScriptNamePart for provider'
        . ' #ProviderAlias contains illegal characters that are not allowed for Unity shops.'
        . ' Allowed are only: a-z, A-Z, 0-9, - and _.',
        'ShopRefAlias_IllegalStartEnd' => 'The WebServerScriptNamePart #ShopRefWebServerScriptNamePart for provider'
        . ' #ProviderAlias must not start or end with a hyphen (-) or an underscore (_).'
    }
}

use constant SHOP_REF_CLASS => "ShopRefClass";

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Provider->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'Provider' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $Provider->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Provider = DE_EPAGES::ShopConfiguration::API::Object::Provider->insert($hInfo);
# example      my $Class = LoadClassByAlias('Provider');
#               my $Provider = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertProvider' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $Provider | new object | object
# hook         OBJ_InsertProvider | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProviderID    - provider id          - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertProvider', {'Object' => $self, 'ProviderID' => $self->id });
    delete $hInfo->{$_} foreach qw ( User ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Provider->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteProvider' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteProvider | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProviderID    - provider id          - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteProvider', {'Object' => $self, 'ProviderID' => $self->id });
    DeleteProviderStore($_) foreach @{GetPKeyProviderStoresByProviderID($self->id)};
    $self->set({
        'User' => undef,
    });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Provider->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateProvider' and 'OBJ_AfterUpdateProvider'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateProvider | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProviderID    - provider id          - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateProvider | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProviderID    - provider id          - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;
    TriggerHook('OBJ_BeforeUpdateProvider', {'Object'=>$self, 'Values'=> $hValues, 'ProviderID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateProvider',  {'Object'=>$self, 'Values'=> $hValues, 'ProviderID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     exportXMLElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->exportXMLElements($Driver, $hAllAttributes);
#----------------------------------------------------------------------------------------
# description  Exports xml data of multilinguagal attributes of this object.
#----------------------------------------------------------------------------------------
# input        $Driver | export driver | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $allAttributes) = @_;
    $Driver->ProviderStoresByProvider();
    $self->SUPER::exportXMLElements($Driver, $allAttributes);
}

#========================================================================================
# function     hasStore
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HasStore = $Provider->hasStore( $Store )
# example      if( $Provider->hasStore( $Store ) { ... }
#----------------------------------------------------------------------------------------
# description  Returns true if the store database is assigned to the provider.
#----------------------------------------------------------------------------------------
# input        $Store | store object | object
# return       $HasStore | true if the provider has the store | boolean
#========================================================================================
sub hasStore {
    my $self = shift;
    my ($Store) = @_;

    return 0 unless ExistsProviderStore( $Store->id );
    return GetInfoProviderStore( $Store->id )->{ProviderID} == $self->id ? 1 : 0;
}

#========================================================================================
# function     addStore
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Provider->addStore( $Store )
#----------------------------------------------------------------------------------------
# description  Assigns a store database to the provider.
#----------------------------------------------------------------------------------------
# input        $Store | store object | object
#========================================================================================
sub addStore {
    my $self = shift;
    my ($Store) = @_;

    DeleteProviderStore( $Store->id ) if ExistsProviderStore( $Store->id );

    InsertProviderStore( {
        StoreID => $Store->id,
        ProviderID => $self->id,
        IsDefault => 0 } );
    return;
}

#========================================================================================
# function     removeStore
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Provider->removeStore( $Store )
#----------------------------------------------------------------------------------------
# description  Unassigns a store database from the provider. The database is not deleted.
#----------------------------------------------------------------------------------------
# input        $Store | store object | object
#========================================================================================
sub removeStore {
    my $self = shift;
    my ($Store) = @_;

    DeleteProviderStore( $Store->id );
    return;
}

#========================================================================================
# function     stores
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aStores = $Provider->stores
#----------------------------------------------------------------------------------------
# description  Returns all store databases that are assigned to the provider.
#----------------------------------------------------------------------------------------
# return       $aStores | list of Store objects | ref.array.Store
#========================================================================================
sub stores {
    my $self = shift;

    return [
        map { LoadObject($_) }
        @{GetPKeyProviderStoresByProviderID( $self->id )}
    ];
}

#========================================================================================
# function     databases
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aDatabases = $Provider->databases
#----------------------------------------------------------------------------------------
# description  Returns all store databases that are assigned to the provider.
#----------------------------------------------------------------------------------------
# return       $aDatabases | list of Store objects | ref.array.Store
#========================================================================================
sub databases {
    my $self = shift;

    return $self->stores;
}

#========================================================================================
# function     features
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aFeatures = $Provider->features
#----------------------------------------------------------------------------------------
# description  Returns all features that are available for a provider for configuration
#               of feature collections. This list contains the maximum of all features
#               of all assigned store databases.
#----------------------------------------------------------------------------------------
# return       $aFeatures | list of feature objects | ref.array.object
#========================================================================================
sub features {
    my $self = shift;

    my %Features;
    foreach my $Store ( @{ $self->get('Stores') } ) {
        foreach my $FeatureRef ( @{ $Store->get('Features') } ) {
            $Features{$FeatureRef->id} = $FeatureRef unless exists $Features{$FeatureRef->id};
        }
    }

    return [ values %Features ];
}

#========================================================================================
# function     featurePacks
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aFeaturePacks = $Provider->featurePacks
#----------------------------------------------------------------------------------------
# description  Returns all feature packs of a provider.
#----------------------------------------------------------------------------------------
# return       $aFeaturePacks | list of feature pack objects | ref.array.object
#========================================================================================
sub featurePacks {
    my $self = shift;

    my $ClassFeatureCollection = LoadClassByAlias('FeatureCollection');
    return [
        sort { $a->get('Position') <=> $b->get('Position') }
        grep { $_->instanceOf('FeaturePack') }
        @{ $ClassFeatureCollection->getObjects('ByProviderID', [$self->id]) }
    ];
}

#========================================================================================
# function     featurePack
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeaturePack = $Provider->featurePack( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns a feature pack of a provider.
#----------------------------------------------------------------------------------------
# input        $Alias | feature pack alias | string
# return       $FeaturePack | feature pack object | object
#========================================================================================
sub featurePack {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'Alias', $Alias );

    return $self->folder('FeaturePacks')->child( $Alias );
}

#========================================================================================
# function     existsFeaturePack
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Provider->existsFeaturePack( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns true if the provider has a feature pack with the given alias name.
#----------------------------------------------------------------------------------------
# input        $Alias | feature pack alias | string
# return       $Exists | true if the feature pack esists | boolean
#========================================================================================
sub existsFeaturePack {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'Alias', $Alias );

    return $self->folder('FeaturePacks')->existsChild( $Alias );
}

#========================================================================================
# function     insertFeaturePack
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeaturePack = $Provider->insertFeaturePack( $hFeaturePack );
#----------------------------------------------------------------------------------------
# description  Inserts a new feature pack for the provider.
#----------------------------------------------------------------------------------------
# input        $hFeaturePack | feature pack attributes, required: Alias | ref.hash
# return       $FeaturePack | feature pack object | object
#========================================================================================
sub insertFeaturePack {
    my $self = shift;
    my ( $hFeaturePack ) = @_;

    return $self->folder('FeaturePacks')->insertChild( 'FeaturePack', {
        %$hFeaturePack,
        Provider => $self,
    } );
}

#========================================================================================
# function     shopService
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopService = $self->shopService;
# syntax       $ShopService = $self->shopService( $Store );
#----------------------------------------------------------------------------------------
# description  Returns a web service client of the ShopService.
#----------------------------------------------------------------------------------------
# input        $Store | (optional) Store object | object
# return       $ShopService | web service client | object
#========================================================================================
sub shopService {
    my $self = shift;
    my ($Store) = @_;
    $Store //= $self->store;

    my $soap = DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient->new(
        'store' => $Store,
        'uri' => 'urn://epages.de/WebService/ShopService/2004/08'
        );

    return $soap;
}

#========================================================================================
# function     shopTypes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aShopTypes = $Provider->shopTypes
#----------------------------------------------------------------------------------------
# description  Returns all shop types packs of a provider.
#----------------------------------------------------------------------------------------
# return       $aShopTypes | list of shop type objects | ref.array.object
#========================================================================================
sub shopTypes {
    my $self = shift;

    my $ClassFeatureCollection = LoadClassByAlias('FeatureCollection');
    return [
        sort { $a->get('Position') <=> $b->get('Position') }
        grep { $_->instanceOf('ShopType') }
        @{ $ClassFeatureCollection->getObjects('ByProviderID', [$self->id]) }
    ];
}

#========================================================================================
# function     shopType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopType = $Provider->shopType( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns a shop type of a provider.
#----------------------------------------------------------------------------------------
# input        $Alias | shop type alias | string
# return       $ShopType | shop type object | object
#========================================================================================
sub shopType {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'Alias', $Alias );

    return $self->folder('ShopTypes')->child( $Alias );
}

#========================================================================================
# function     shopRefClass
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopRefClass = $Provider->shopRefClass();
#----------------------------------------------------------------------------------------
# description  Returns a shop reference class of a provider with own
#               shop ref class different to system shop ref class.
#               create that class unless its exists
#----------------------------------------------------------------------------------------
# return       $ShopRefClass | shop reference class object | object
#========================================================================================
sub shopRefClass {
    my $self = shift;
    unless ( $self->existsChild(SHOP_REF_CLASS)) {
        $self->insertChild( 'Class', {
            Alias => SHOP_REF_CLASS,
            BaseClass => LoadClassByAlias( 'ShopRef' ),
        })
    }
    return $self->child(SHOP_REF_CLASS);
}

#========================================================================================
# function     existsShopType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Provider->existsShopType( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns true if the provider has a shop type with the given alias name.
#----------------------------------------------------------------------------------------
# input        $Alias | shop type alias | string
# return       $Exists | true if the shop type esists | boolean
#========================================================================================
sub existsShopType {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'Alias', $Alias );

    return $self->folder('ShopTypes')->existsChild( $Alias );
}

#========================================================================================
# function     insertShopType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopType = $Provider->insertShopType( $hShopType );
#----------------------------------------------------------------------------------------
# description  Inserts a new shop type for the provider.
#----------------------------------------------------------------------------------------
# input        $hShopType | shop type attributes, required: Alias | ref.hash
# return       $ShopType | shop type object | object
#========================================================================================
sub insertShopType {
    my $self = shift;
    my ( $hShopType ) = @_;

    return $self->folder('ShopTypes')->insertChild( 'ShopType', {
        %$hShopType,
        Provider => $self,
    } );
}

#========================================================================================
# function     shop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopRef = $Provider->shop( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns a shop of the provider.
#----------------------------------------------------------------------------------------
# input        $Alias | unique shop alias | string
# return       $ShopRef | shop reference object | object
#========================================================================================
sub shop {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'Alias', $Alias );

    return $self->folder('ShopRefs')->child( $Alias );
}

#========================================================================================
# function     existsShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Provider->existsShop( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns true a shop with the given alias name exists for the provider.
#               Thereby small and capital letters are identified with each other
#               using the search functionality for the shop alias.
#----------------------------------------------------------------------------------------
# input        $Alias | unique shop alias | string
# return       $Exists | true if the shop exists | boolean
#========================================================================================
sub existsShop {
    my $self = shift;
    my ($Alias) = @_;
    TestString( 'Alias', $Alias );

    # escape undercore (_) and percent (%) to avoid them to be used as SQL wildcards
    $Alias =~ s/_/\\_/;
    $Alias =~ s/%/\\%/;

    my $ClassShopRef = LoadClassByAlias('ShopRef');
    my $SearchDefinition = GetSearchDefinition($ClassShopRef, 'ShopRef');

    my %Parameters = (
        'Provider.ID' => $self->id,
        'Alias' => $Alias
     );

    my $Pager = DE_EPAGES::Search::API::Object::PagerSearch->new({
        'ObjectID'  => $self->id,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => \%Parameters
    });

    return scalar @{$Pager->allItemIDs} ? 1 : 0;
}

#========================================================================================
# function     insertShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopRef = $Provider->insertShop( $hShop )
# example      $ShopRef = $Provider->insertShop({
#                   ShopType    => $ShopType,
#                   Store       => $Store,
#                   Alias       => $Alias,
#                   ShopAlias   => $ShopAlias,
#                   IsClosed    => $IsClosed,
#                   DomainName  => $DomainName,
#                   HasSSLCertificate => $HasSSLCertificate,
#                   WebServerScriptNamePart => $WebServerScriptNamePart,
#                   IsTrialShop => $IsTrialShop,
#                   IsInternalTestShop => $IsInternalTestShop,
#                   ImportFiles => $aImportFiles,
#                   PublicFilesDir => $PublicFilesDir,
#               });
#----------------------------------------------------------------------------------------
# description  Inserts a shop of the provider.
#               Re-uses a pre-created shop if one exists in the given store database
#               with the given shop type. In this case, the parameters ImportFiles and
#               PublicFilesDir are ignored.
#               See also <function preCreateShop>.
#----------------------------------------------------------------------------------------
# input        $hShop | parameters for the new shop
#               <ul>
#               <li>$ShopType - shop type - object
#               <li>$Store - store database - object
#               <li>$Alias - unique shop name for the provider - string
#               <li>$ShopAlias - unique shop name for the database - string
#               <li>$IsClosed - true if the new shop is closed by provider (optional, default: 0) - boolean
#               <li>$DomainName - domain name to access the shop (optional) - string
#               <li>$HasSSLCertificate - true if the web server has an SSL certificate for $DomainName (optional) - boolean
#               <li>$WebServerScriptNamePart - script name part of the shop (optional) - string
#               <li>$IsTrialShop - true if the shop is in trial period (optional, default: 0) - boolean
#               <li>$IsInternalTestShop - true if the shop is used for test or support purposes (optional, default: 0) - boolean
#               <li>$aImportFiles - list of import files contianing initial shop date. These
#                       files must be accessible for the application servers that are
#                       responsible for the specified database. (optional) - string
#               <li>$PublicFilesDir - directory on the application server where image files
#                       for the new shop are located. These files are copied to the
#                       shop directory. (optional) - string
#               </ul> | ref.hash
# return       $ShopRef | shop reference object | object
#========================================================================================
sub insertShop {
    my $self = shift;
    my ($hShop) = @_;

    # test if the shopref alias already exists
    Error('ShopRefAlias_Exists', {
        ShopRefAlias => $hShop->{Alias},
        ProviderAlias => $self->alias
    }) if $self->existsShop( $hShop->{Alias} );

    if ($hShop->{'ShopType'}->get('HasUnityEnabled')) {
        Error('ShopRefAlias_ContainsIllegalCharacters', {
            ShopRefWebServerScriptNamePart => $hShop->{WebServerScriptNamePart},
            ProviderAlias => $self->alias
        }) if ( $hShop->{WebServerScriptNamePart} !~ m/^[a-zA-Z0-9_-]+$/);
        Error('ShopRefAlias_IllegalStartEnd', {
            ShopRefWebServerScriptNamePart => $hShop->{WebServerScriptNamePart},
            ProviderAlias => $self->alias
        }) if ( $hShop->{WebServerScriptNamePart} =~ m/^-|^_|-$|_$/ );
    }

    # try to reuse a pre-created shop or else create a new one
    my $ShopRef = $self->_getPreCreatedShop( $hShop );
    $ShopRef = $self->_createShop( $hShop ) unless defined $ShopRef;

    # register Unity
    eval {
        $ShopRef->registerUnity;
    };
    if (ExistsError()) {
        my $Error = GetError();

        eval { $ShopRef->delete; };
        LogError(GetError()) if ExistsError();

        Error($Error);
    }

    return $ShopRef;
}

#========================================================================================
# function     preCreateShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopRef = $Provider->preCreateShop({
#                   ShopType    => $ShopType,
#                   Store       => $Store,
#                   ImportFiles => $aImportFiles,
#                   PublicFilesDir => $PublicFilesDir,
#               });
#----------------------------------------------------------------------------------------
# description  Creates a new shop without a real shop alias, which can be re-used
#               later by <function insertShop>
#----------------------------------------------------------------------------------------
# input        $hShop | parameters for the new shop
#               <ul>
#               <li>$ShopType - shop type - object
#               <li>$Store - store database - object
#               <li>$aImportFiles - list of import files contianing initial shop date. These
#                       files must be accessible for the application servers that are
#                       responsible for the specified database. (optional) - string
#               <li>$PublicFilesDir - directory on the application server where image files
#                       for the new shop are located. These files are copied to the
#                       shop directory. (optional) - string
#               </ul> | ref.hash
# return       $ShopRef | shop reference object | object
#========================================================================================
sub preCreateShop {
    my $self = shift;
    my ($hShop) = @_;

    my $Alias = $self->_getNextPreCreatedShopAlias;

    # test if we have already the precreated alias
    while ($self->folder('ShopRefs')->existsChild($Alias)) {
        $Alias = $self->_getNextPreCreatedShopAlias;
    }

    $hShop->{'Alias'} = $Alias;
    $hShop->{'ShopAlias'} = $Alias;
    delete $hShop->{'DomainName'}; # just in case

    my $ShopRef = $self->_createShop( $hShop );

    # make it available for new shop creation when this shop is complete
    eval {
        $ShopRef->set({
            IsClosed => 1,
            IsPreCreated => 1,
        });
    };
    if( ExistsError() ) {
        my $Error = GetError();

        GetLog()->debug( 'Delete pre-created shop after error: ' . $Error->message );
        $ShopRef->delete;

        Error( $Error );
    }

    return $ShopRef;
}

# returns the shop alias of the next pre-created shop
sub _getNextPreCreatedShopAlias {
    my $self = shift;

    # attribute is self incrementing
    my $Number = $self->get('NextPreCreatedShop');
    return 'PreCreated'.$Number;
}

# create a new shop, do not reuse a pre-created shop
sub _createShop {
    my $self = shift;
    my ($hShop) = @_;

    my $ShopRefClass = LoadClassByAlias('ShopRef');

    # attributes for insertChild()
    my %ShopRef = (
        Provider  => $self,
        IsClosed  => 1, # will be set later in RemoteShop->create
    );

    # attributes for set()
    my %ShopRefAttr = ();

    # parameters for createRemoteShop()
    my %RemoteShop = (
        Store => $hShop->{'Store'}
    );

    # attributes for $RemoteShop->set()
    my %RemoteValues;

    foreach my $Key ( keys %$hShop ) {
        # UNITY-6210 set WebServerScriptNamePart in createShop call as well
        # It has to be set in createRemoteShop and in remoteShop->set again
        # to work around missing pooldb entries.
        if (ElementOf( $Key, qw(WebServerScriptNamePart) ) ) {
            $RemoteShop{$Key} = $hShop->{$Key};
        }
        if( $ShopRefClass->existsAttribute($Key) ) {
            my $Attribute = $ShopRefClass->attribute($Key);
            if( $Attribute->get('Package') eq 'DE_EPAGES::ShopConfiguration::API::Attributes::ShopRef_Remote' ) {
                $RemoteValues{$Key} = $hShop->{$Key};
            }
            elsif ( ElementOf( $Key, qw(IsClosed IsInternalTestShop IsTrialShop) ) )
            {
                $ShopRefAttr{$Key} = $hShop->{$Key};
            }
            else {
                $ShopRef{$Key} = $hShop->{$Key};
            }
        }
        else {
            $RemoteShop{$Key} = $hShop->{$Key};
        }
    }

    my $ShopRef = $self->folder('ShopRefs')->insertChild($ShopRefClass , \%ShopRef);

    eval {
        $ShopRef->createRemoteShop(\%RemoteShop);
        $ShopRef->set(\%ShopRefAttr);
        $ShopRef->remoteShop->set(\%RemoteValues) if scalar keys %RemoteValues;
    };
    if (ExistsError()) {
        my $Error = GetError();

        eval { $ShopRef->delete; };
        LogError(GetError()) if ExistsError();

        Error($Error);
    }

    return $ShopRef;
}

# create a new shop, using a pre-created shop
# returns undef if no matching pre-created shop exists
sub _getPreCreatedShop {
    my $self = shift;
    my ($hShop) = @_;

    my $ShopRef = GetPreCreatedShop( $self, $hShop->{Store}, $hShop->{ShopType} );
    return undef unless defined $ShopRef;

    my $SystemTimeZone = LoadRootObject()->get('TimeZone');

    eval {
        my $RemoteShop = $ShopRef->remoteShop;
        my $OldAlias = $RemoteShop->get(['Alias'])->{Alias};
        GetLog()->debug( "using precreated shop $OldAlias" );

        my $CreationDate =  GetCurrentDateTime();

        my $parser = DE_EPAGES::XML::API::SOAPDateTimeFormatXML->new();
        my $CreationDateString  = $parser->format_datetime($CreationDate, $SystemTimeZone);
        my %RemoteValues = (
            Alias => $hShop->{ShopAlias},
            CreationDate  => $CreationDateString,
        );
        $RemoteShop->set( \%RemoteValues );

        my $ShopRefClass = LoadClassByAlias('ShopRef');
        my %ShopRef = map { $_ => $hShop->{$_} } grep { $ShopRefClass->existsAttribute($_) } keys %$hShop;
        $ShopRef{'IsPreCreated'} = 0;
        $ShopRef{'CreationDate'} = $CreationDate;
        $ShopRef{'IsClosed'} = $hShop->{'IsClosed'};

        $ShopRef->set({ StoreObjectPath => '/' . JoinObjectPath([ 'Shops', $hShop->{ShopAlias} ]) });
        $ShopRef->set( \%ShopRef );

        GetLog()->debug( "renamed pre-created shop '$OldAlias' to '$hShop->{ShopAlias}'" );
    };
    if( ExistsError() ) {
        my $Error = GetError();
        GetLog()->debug( $Error->longMessage );

        # delete the shop on error
        GetLog()->debug( "delete shop '$hShop->{ShopAlias}' after error" );
        eval {
            $ShopRef->delete;
        };
        LogError(GetError()) if ExistsError();
        Error($Error);
    }

    return $ShopRef;
}

#========================================================================================
# function     shops
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aShopRefs = $Provider->shops;
# example      $_->delete foreach @{ $Provider->shops };
#----------------------------------------------------------------------------------------
# description  Returns all shops of a provider.
#----------------------------------------------------------------------------------------
# return       $aShopRefs | list of shop reference objects | ref.array.object
#========================================================================================
sub shops {
    my $self = shift;
    return $self->folder( 'ShopRefs' )->children;
}

#========================================================================================
# function     userss
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aUsers = $Provider->users;
# example      $_->delete foreach @{ $Provider->users };
#----------------------------------------------------------------------------------------
# description  Returns all users of a provider.
#----------------------------------------------------------------------------------------
# return       $aUsers | list of user reference objects | ref.array.object
#========================================================================================
sub users {
    my $self = shift;
    return $self->folder( 'Users' )->children;
}

#========================================================================================
# function     existsSiteLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Site->existsSiteLocale($LocaleID);
# example      if( $Site->existsSiteLocale('en_GB') ) { ... }
#----------------------------------------------------------------------------------------
# description  Returns true if the site supports the given locale.
#----------------------------------------------------------------------------------------
# input        $LocaleID | locale id, e.g. 'en_GB' | string
# return       $Exists | true if the locale is supported by the site | boolean
#========================================================================================
sub existsSiteLocale {
    my $self = shift;
    my ($LocaleID) = @_;

    return LoadRootObject()->existsSiteLocale($LocaleID);
}

#========================================================================================
# function     siteLocale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Locale = $Site->siteLocale($LocaleID);
# example      $Site->siteLocale('en_GB')->roundMoney($amount, $CurrencyID);
#----------------------------------------------------------------------------------------
# description  Returns locale object with given locale identifier.
#----------------------------------------------------------------------------------------
# input        $LocaleID | locale id, e.g. 'en_GB' | string
# return       $Locale | supported locale of site | object
#========================================================================================
sub siteLocale {
    my $self = shift;
    my ($LocaleID) = @_;

    return LoadRootObject()->siteLocale($LocaleID);
}

1;
