#========================================================================================
# package      DE_EPAGES::Test::API::TranslationFiles
# base         DE_EPAGES::Test::API::UnitTest
# state        public
#----------------------------------------------------------------------------------------
# description  Checks translation files for correct file structure
#========================================================================================
package DE_EPAGES::Test::API::TranslationFiles;
use base qw (Exporter DE_EPAGES::Test::API::UnitTest);

use strict;

use Test::More;
use DE_EPAGES::Core::API::File qw (
    GetFileNames
    GetFileContentLines
    GetSubDirectories
);

our @EXPORT_OK = qw(
    TestCartridges
    TestCompany
    TestTranslationFile
);

sub test {
    my $self = shift;

    # get companies
    my @Companies = _GetCompanies();

    foreach my $Company (@Companies) {
        TestCompany($Company);
    }

    done_testing();
}

#========================================================================================
# function     TestCompany
#----------------------------------------------------------------------------------------
# syntax       TestCompany($Company);
#----------------------------------------------------------------------------------------
# description  Tests all translation files of a company
#----------------------------------------------------------------------------------------
# input        $Company     | cartridge folder name, e.g. DE_EPAGES | string
#========================================================================================
sub TestCompany {
    my ($Company) = @_;

    # get cartridges
    my @Cartridges = _GetCartridges($Company);

    TestCartridges(\@Cartridges, $Company);
}

#========================================================================================
# function     TestCartridges
#----------------------------------------------------------------------------------------
# syntax       TestCartridges($aCartridges, $Company);
#----------------------------------------------------------------------------------------
# description  Tests all translation files of a cartridge
#----------------------------------------------------------------------------------------
# input        $aCartridges | list of cartridge names | ref.array
# input        $Company     | cartridge folder name, e.g. DE_EPAGES | string
#========================================================================================
sub TestCartridges {
    my ($aCartridges, $Company) = @_;

    # get Translation*.xml files
    my @TranslationFiles = map { _GetTranslationFiles($_, $Company) } @$aCartridges;

    foreach my $FileName (@TranslationFiles) {
        TestTranslationFile($FileName);
    }
}


#========================================================================================
# function     TestTranslationFile
#----------------------------------------------------------------------------------------
# syntax       TestTranslationFile($FileName);
#----------------------------------------------------------------------------------------
# description  Tests a single translation file
#----------------------------------------------------------------------------------------
# input        $FileName | file name (full path) | string
#========================================================================================
sub TestTranslationFile {
    my ($FileName) = @_;

    my $aFileContentLines = GetFileContentLines($FileName, {'encoding' => 'utf-8'});
    # check XM declaration
    is($aFileContentLines->[0], "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n",
        $FileName. ' does start with the correct xml declaration: ' . $aFileContentLines->[0]);

    # check <epages> tags
    ok($aFileContentLines->[1] =~ m/^<(ns0:|)epages( |>)/,
        $FileName. ' does have the opening <epages> tag: ' . $aFileContentLines->[1]);
    ok($aFileContentLines->[-1] =~ m/^<\/(ns0:|)epages>\n/,
        $FileName. ' does have the closing </epages> tag: ' . $aFileContentLines->[-1]);

    # check <Language> tags
    ok($aFileContentLines->[2] =~ m/^  (<(ns0:|)Language Language="[a-z]{2}">)\n/
        || $aFileContentLines->[3] =~ m/^  (<(ns0:|)Language Language="[a-z]{2}">)\n/,
        $FileName. ' does have a correct opening <Language> tag: ' . $aFileContentLines->[2]);
    ok($aFileContentLines->[-2] =~ m/^  <\/(ns0:|)Language>\n/,
        $FileName. ' does have a correct closing </Language> tag: ' . $aFileContentLines->[-2]);

    # check for singleton <Translation> tags
    my $FileContent = join '', @$aFileContentLines;
    ok($FileContent !~ m/(<Translation[^>]*\/>)/,
        $FileName. ' contains no singleton <Translation /> tag: ' . $1);

    # check for singleton <notrans /> tags
    ok($FileContent !~ m/(<notrans *\/>)/,
        $FileName. ' contains no singleton <notrans /> tag: ' . $1);

    # check for untrimmed text
    ok($FileContent !~ m/<Translation[^>]*>(\s+[^\s<>]+.*)<\/Translation>/
        && $FileContent !~ m/<Attribute[^>]*>(\s+[^\s<>]+.*)<\/Attribute>/,
        $FileName. ' has no leading whitespaces: ' . $1);
    ok($FileContent !~ m/<Translation[^>]*>(.*[^\s<>]+\s+)<\/Translation>/
        && $FileContent !~ m/>Attribute[^>]*>(.*[^\s<>]+\s+)<\/Attribute>/,
        $FileName. ' has no trailing whitespaces: ' . $1);

    # check for XML junk
    ok($FileContent !~ m/<\/Translation>(\s*[^\s<>]+\s*)</
        && $FileContent !~ m/<\/Attribute>(\s*[^\s<>]+\s*)</,
        $FileName. ' has no junk between XML tags: ' . $1);
    ok($FileContent !~ m/[\x{007f}-\x{009f}]/,
        $FileName. ' does not include awkward whitespaces');
    ok($FileContent !~ m/<!\[CDATA\[/,
        $FileName. ' does not include \'<![CDATA[\' tags');

    # check for wrong indentations
    ok($FileContent !~ m/\n {0,3}<Translation/ && $FileContent !~ m/\n {5,}<Translation/,
        $FileName. ' has no wrong aligned <Translation> tag');
    ok($FileContent !~ m/\n {0,3}<Object/ && $FileContent !~ m/\n {5,}<Object/,
        $FileName. ' has no wrong aligned <Object> tag');
    ok($FileContent !~ m/\n {0,3}<\/Object/ && $FileContent !~ m/\n {5,}<\/Object/,
        $FileName. ' has no wrong aligned </Object> tag');
    ok($FileContent !~ m/\n {0,3}<Attribute/ && $FileContent !~ m/\n {5}<Attribute/ &&
        $FileContent !~ m/\n {7,}<Attribute/,
        $FileName. ' has no wrong aligned <Attribute> tag');
    ok($FileContent !~ m/\n {0,3}<\/Attribute/ && $FileContent !~ m/\n {5}<\/Attribute/ &&
        $FileContent !~ m/\n {7,}<\/Attribute/,
        $FileName. ' has no wrong aligned </Attribute> tag');
    ok($FileContent !~ m/\t/, $FileName. ' contains no tabulator');
}

#----------------------------------------------------------------------------------------
# private helper
#----------------------------------------------------------------------------------------
sub _GetTranslationFiles {
    my ($CartridgeName, $Company) = @_;

    my @Dirlist = (
        join ('/', $ENV{'EPAGES_CARTRIDGES'}, $Company, $CartridgeName, 'Templates'),
        join ('/', $ENV{'EPAGES_CARTRIDGES'}, $Company, $CartridgeName, 'Database'),
        join ('/', $ENV{'EPAGES_CARTRIDGES'}, $Company, $CartridgeName, 'Data'),
        );

    # file exceptions that seem to be translation files but are not
    my @FileNameExceptions = (
        '/srv/epages/eproot/Cartridges/DE_EPAGES/Forum/Templates/RSS/RSS-Blog.Page.xml',
        '/srv/epages/eproot/Cartridges/DE_EPAGES/Forum/Templates/RSS/RSS-Forum.Page.xml',
        '/srv/epages/eproot/Cartridges/DE_EPAGES/Object/Database/XML/Languages.en-US.xml',
        '/srv/epages/eproot/Cartridges/DE_EPAGES/Object/Database/XML/Languages.es-MX.xml',
        '/srv/epages/eproot/Cartridges/DE_EPAGES/Object/Database/XML/Languages.pt-BR.xml'
    );

    return grep { !($_ ~~ @FileNameExceptions) }
           grep { /\.[^\.]*\.xml$/x }
           map  { @{GetFileNames($_, {'recursive' => 1})} }
           grep { -d $_ } @Dirlist;
}

sub _GetCartridges {
    my ($Company) = @_;

    my $aCartridges = GetSubDirectories($ENV{'EPAGES_CARTRIDGES'} . '/' . $Company, {'relative' => 1});
    my @ExcludedCartridges;
    @ExcludedCartridges = (
        'DocdataPay',
        'ELogistics',
        'Envialia',
        'TagCommander',
        'MultiStore',
        'MultiStoreCommon',
        'MultiStoreConnector',
        'MultiStoreMaster',
        'MultiStoreMasterConnector',
        'MultiStoreProvider',
        'MultiStoreShipping'
    ) if $Company eq 'DE_EPAGES';

    return grep { !($_ ~~ @ExcludedCartridges) } @$aCartridges;
}

sub _GetCompanies {
    my $aCompanies = GetSubDirectories($ENV{'EPAGES_CARTRIDGES'}, {'relative' => 1});

    my @ExcludedCompanies = (
        'CPAN',
        'RND_EPAGES',
        '.git'
    );

    return grep { !($_ ~~ @ExcludedCompanies) } @$aCompanies;
}

1;
